@@ -4,13 +4,15 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: channels.c 1.46 2005/09/11 14:22:24 kls Exp $
+ * $Id: channels.c 1.51 2006/04/17 12:18:57 kls Exp $
  */
 
 #include "channels.h"
 #include <linux/dvb/frontend.h>
 #include <ctype.h>
 #include "device.h"
+#include "epg.h"
+#include "timers.h"
 
 // IMPORTANT NOTE: in the 'sscanf()' calls there is a blank after the '%d'
 // format characters in order to allow any number of blanks after a numeric
@@ -174,6 +176,7 @@ cChannel::cChannel(void)
   guard        = GUARD_INTERVAL_AUTO;
   hierarchy    = HIERARCHY_AUTO;
   modification = CHANNELMOD_NONE;
+  schedule     = NULL;
   linkChannels = NULL;
   refChannel   = NULL;
 }
@@ -184,6 +187,7 @@ cChannel::cChannel(const cChannel &Channel)
   shortName = NULL;
   provider = NULL;
   portalName = NULL;
+  schedule     = NULL;
   linkChannels = NULL;
   refChannel   = NULL;
   *this = Channel;
@@ -245,6 +249,15 @@ int cChannel::Transponder(void) const
   return tf;
 }
 
+bool cChannel::HasTimer(void) const
+{
+  for (cTimer *Timer = Timers.First(); Timer; Timer = Timers.Next(Timer)) {
+      if (Timer->Channel() == this)
+         return true;
+      }
+  return false;
+}
+
 int cChannel::Modification(int Mask)
 {
   int Result = modification & Mask;
@@ -280,6 +293,9 @@ bool cChannel::SetSatTransponderData(int Source, int Frequency, char Polarizatio
   // Sometimes the transponder frequency is set to 0, which is just wrong
   if (Frequency == 0)
      return false;
+  // Sometimes the symbol rate is off by one
+  if (abs(srate - Srate) <= 1)
+     Srate = srate;
 
   if (source != Source || frequency != Frequency || polarization != Polarization || srate != Srate || coderateH != CoderateH) {
      if (Number()) {
@@ -293,6 +309,7 @@ bool cChannel::SetSatTransponderData(int Source, int Frequency, char Polarizatio
      srate = Srate;
      coderateH = CoderateH;
      modulation = QPSK;
+     schedule = NULL;
      }
   return true;
 }
@@ -310,6 +327,7 @@ bool cChannel::SetCableTransponderData(int Source, int Frequency, int Modulation
      modulation = Modulation;
      srate = Srate;
      coderateH = CoderateH;
+     schedule = NULL;
      }
   return true;
 }
@@ -331,6 +349,7 @@ bool cChannel::SetTerrTransponderData(int Source, int Frequency, int Bandwidth,
      coderateL = CoderateL;
      guard = Guard;
      transmission = Transmission;
+     schedule = NULL;
      }
   return true;
 }
@@ -350,6 +369,7 @@ void cChannel::SetId(int Nid, int Tid, int Sid, int Rid)
      rid = Rid;
      if (Number())
         Channels.HashChannel(this);
+     schedule = NULL;
      }
 }
 
@@ -586,7 +606,7 @@ static const char *ParseParameter(const char *s, int &Value, const tChannelParam
            return p;
         }
      }
-  esyslog("ERROR: illegal value for parameter '%c'", *(s - 1));
+  esyslog("ERROR: invalid value for parameter '%c'", *(s - 1));
   return NULL;
 }
 
