@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 1.235 2005/11/11 13:22:02 kls Exp $
+ * $Id: config.h 1.255 2006/04/29 09:24:07 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -19,8 +19,21 @@
 #include "i18n.h"
 #include "tools.h"
 
-#define VDRVERSION  "1.3.37"
-#define VDRVERSNUM   10337  // Version * 10000 + Major * 100 + Minor
+// VDR's own version number:
+
+#define VDRVERSION  "1.4.0"
+#define VDRVERSNUM   10400  // Version * 10000 + Major * 100 + Minor
+
+// The plugin API's version number:
+
+#define APIVERSION  "1.4.0"
+#define APIVERSNUM   10400  // Version * 10000 + Major * 100 + Minor
+
+// When loading plugins, VDR searches them by their APIVERSION, which
+// may be smaller than VDRVERSION in case there have been no changes to
+// VDR header files since the last APIVERSION. This allows compiled
+// plugins to work with newer versions of the core VDR as long as no
+// VDR header files have changed.
 
 #define MAXPRIORITY 99
 #define MAXLIFETIME 99
@@ -61,20 +74,6 @@ public:
   bool Accepts(in_addr_t Address);
   };
 
-#define CACONFBASE 100
-
-class cCaDefinition : public cListObject {
-private:
-  int number;
-  char *description;
-public:
-  cCaDefinition(void);
-  ~cCaDefinition();
-  bool Parse(const char *s);
-  int Number(void) const { return number; }
-  const char *Description(void) const { return description; }
-  };
-
 template<class T> class cConfig : public cList<T> {
 private:
   char *fileName;
@@ -166,15 +165,9 @@ public:
   bool Acceptable(in_addr_t Address);
   };
 
-class cCaDefinitions : public cConfig<cCaDefinition> {
-public:
-  const cCaDefinition *Get(int Number);
-  };
-
 extern cCommands Commands;
 extern cCommands RecordingCommands;
 extern cSVDRPhosts SVDRPhosts;
-extern cCaDefinitions CaDefinitions;
 
 class cSetupLine : public cListObject {
 private:
@@ -210,8 +203,10 @@ public:
   char OSDTheme[MaxThemeName];
   int PrimaryDVB;
   int ShowInfoOnChSwitch;
+  int TimeoutRequChInfo;
   int MenuScrollPage;
   int MenuScrollWrap;
+  int MenuButtonCloses;
   int MarkInstantRecord;
   char NameInstantRecord[MaxFileName];
   int InstantRecordTime;
@@ -230,7 +225,6 @@ public:
   int EPGLinger;
   int SVDRPTimeout;
   int ZapTimeout;
-  int SortTimers;
   int PrimaryLimit;
   int DefaultPriority, DefaultLifetime;
   int PausePriority, PauseLifetime;
@@ -256,6 +250,8 @@ public:
   int CurrentChannel;
   int CurrentVolume;
   int CurrentDolby;
+  int InitialChannel;
+  int InitialVolume;
   int __EndData__;
   cSetup(void);
   cSetup& operator= (const cSetup &s);
