@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c 1.112 2005/11/26 12:56:09 kls Exp $
+ * $Id: device.c 1.128 2006/04/14 14:34:43 kls Exp $
  */
 
 #include "device.h"
@@ -222,7 +222,7 @@ int cDevice::NextCardIndex(int n)
         esyslog("ERROR: nextCardIndex too big (%d)", nextCardIndex);
      }
   else if (n < 0)
-     esyslog("ERROR: illegal value in IncCardIndex(%d)", n);
+     esyslog("ERROR: invalid value in IncCardIndex(%d)", n);
   return nextCardIndex;
 }
 
@@ -281,28 +281,31 @@ cDevice *cDevice::GetDevice(int Index)
 cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool *NeedsDetachReceivers)
 {
   cDevice *d = NULL;
-  int select = 8, pri;
+  int select = INT_MAX;
 
   for (int i = 0; i < numDevices; i++) {
       bool ndr;
       if (device[i]->ProvidesChannel(Channel, Priority, &ndr)) { // this device is basicly able to do the job
+         int pri;
          if (device[i]->Receiving() && !ndr)
             pri = 0; // receiving and allows additional receivers
-         else if (d && !device[i]->Receiving() && device[i]->ProvidesCa(Channel) < d->ProvidesCa(Channel))
+         else if (!device[i]->Receiving(true) && d && device[i]->ProvidesCa(Channel) < d->ProvidesCa(Channel))
             pri = 1; // free and fewer Ca's
          else if (!device[i]->Receiving() && !device[i]->HasDecoder())
             pri = 2; // free and not a full featured card
+         else if (!device[i]->Receiving() && device[i] != ActualDevice())
+            pri = 3; // free and not the actual device
          else if (!device[i]->Receiving() && !device[i]->IsPrimaryDevice())
-            pri = 3; // free and not the primary device
+            pri = 4; // free and not the primary device
          else if (!device[i]->Receiving())
-            pri = 4; // free
+            pri = 5; // free
          else if (d && device[i]->Priority() < d->Priority())
-            pri = 5; // receiving but priority is lower
+            pri = 6; // receiving but priority is lower
          else if (d && device[i]->Priority() == d->Priority() && device[i]->ProvidesCa(Channel) < d->ProvidesCa(Channel))
-            pri = 6; // receiving with same priority but fewer Ca's
+            pri = 7; // receiving with same priority but fewer Ca's
          else
-            pri = 7; // all others
-         if (pri < select) {
+            pri = 8; // all others
+         if (pri <= select) {
             select = pri;
             d = device[i];
             if (NeedsDetachReceivers)
@@ -322,9 +325,36 @@ void cDevice::Shutdown(void)
       }
 }
 
-bool cDevice::GrabImage(const char *FileName, bool Jpeg, int Quality, int SizeX, int SizeY)
+uchar *cDevice::GrabImage(int &Size, bool Jpeg, int Quality, int SizeX, int SizeY)
 {
-  return false;
+  return NULL;
+}
+
+bool cDevice::GrabImageFile(const char *FileName, bool Jpeg, int Quality, int SizeX, int SizeY)
+{
+  int result = 0;
+  int fd = open(FileName, O_WRONLY | O_CREAT | O_NOFOLLOW | O_TRUNC, DEFFILEMODE);
+  if (fd >= 0) {
+     int ImageSize;
+     uchar *Image = GrabImage(ImageSize, Jpeg, Quality, SizeX, SizeY);
+     if (Image) {
+        if (safe_write(fd, Image, ImageSize) == ImageSize)
+           isyslog("grabbed image to %s", FileName);
+        else {
+           LOG_ERROR_STR(FileName);
+           result |= 1;
+           }
+        free(Image);
+        }
+     else
+        result |= 1;
+     close(fd);
+     }
+  else {
+     LOG_ERROR_STR(FileName);
+     result |= 1;
+     }
+  return result == 0;
 }
 
 void cDevice::SetVideoDisplayFormat(eVideoDisplayFormat VideoDisplayFormat)
@@ -520,6 +550,11 @@ bool cDevice::ProvidesChannel(const cChannel *Channel, int Priority, bool *Needs
   return false;
 }
 
+bool cDevice::IsTunedToTransponder(const cChannel *Channel)
+{
+  return false;
+}
+
 bool cDevice::MaySwitchTransponder(void)
 {
   return !Receiving(true) && !(pidHandles[ptAudio].pid || pidHandles[ptVideo].pid || pidHandles[ptDolby].pid);
@@ -577,10 +612,14 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
   if (LiveView)
      StopReplay();
 
+  // If this card is switched to an other transponder, any receivers still
+  // attached to it ineed to be automatically detached:
+  bool NeedsDetachReceivers = false;
+
   // If this card can't receive this channel, we must not actually switch
   // the channel here, because that would irritate the driver when we
   // start replaying in Transfer Mode immediately after switching the channel:
-  bool NeedsTransferMode = (LiveView && IsPrimaryDevice() && !ProvidesChannel(Channel, Setup.PrimaryLimit));
+  bool NeedsTransferMode = (LiveView && IsPrimaryDevice() && !ProvidesChannel(Channel, Setup.PrimaryLimit, &NeedsDetachReceivers));
 
   eSetChannelResult Result = scrOk;
 
@@ -588,11 +627,14 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
   // use the card that actually can receive it and transfer data from there:
 
   if (NeedsTransferMode) {
-     cDevice *CaDevice = GetDevice(Channel, 0);
+     cDevice *CaDevice = GetDevice(Channel, 0, &NeedsDetachReceivers);
      if (CaDevice && CanReplay()) {
         cStatus::MsgChannelSwitch(this, 0); // only report status if we are actually going to switch the channel
-        if (CaDevice->SetChannel(Channel, false) == scrOk) // calling SetChannel() directly, not SwitchChannel()!
+        if (CaDevice->SetChannel(Channel, false) == scrOk) { // calling SetChannel() directly, not SwitchChannel()!
+           if (NeedsDetachReceivers)
+              CaDevice->DetachAllReceivers();
            cControl::Launch(new cTransferControl(CaDevice, Channel->Vpid(), Channel->Apids(), Channel->Dpids(), Channel->Spids()));
+           }
         else
            Result = scrNoTransfer;
         }
@@ -613,7 +655,7 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
         ciHandler->SetSource(Channel->Source(), Channel->Transponder());
 // Men at work - please stand clear! ;-)
 #ifdef XXX_DO_MULTIPLE_CA_CHANNELS
-        if (Channel->Ca() > CACONFBASE) {
+        if (Channel->Ca() >= CA_ENCRYPTED_MIN) {
 #endif
            ciHandler->AddPid(Channel->Sid(), Channel->Vpid(), 2);
            for (const int *Apid = Channel->Apids(); *Apid; Apid++)
@@ -626,13 +668,15 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
            }
 #endif
         }
+     if (NeedsDetachReceivers)
+        DetachAllReceivers();
      if (SetChannelDevice(Channel, LiveView)) {
         // Start section handling:
         if (sectionHandler) {
            sectionHandler->SetChannel(Channel);
            sectionHandler->SetStatus(true);
            }
-        // Start decrypting any PIDs the might have been set in SetChannelDevice():
+        // Start decrypting any PIDs that might have been set in SetChannelDevice():
         if (ciHandler)
            ciHandler->StartDecrypting();
         }
@@ -661,6 +705,15 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
   return Result;
 }
 
+void cDevice::ForceTransferMode(void)
+{
+  if (!cTransferControl::ReceiverDevice()) {
+     cChannel *Channel = Channels.GetByNumber(CurrentChannel());
+     if (Channel)
+        SetChannelDevice(Channel, false); // this implicitly starts Transfer Mode
+     }
+}
+
 bool cDevice::SetChannelDevice(const cChannel *Channel, bool LiveView)
 {
   return false;
@@ -737,14 +790,19 @@ void cDevice::SetVolume(int Volume, bool Absolute)
      }
 }
 
-void cDevice::ClrAvailableTracks(bool DescriptionsOnly)
+void cDevice::ClrAvailableTracks(bool DescriptionsOnly, bool IdsOnly)
 {
   if (DescriptionsOnly) {
      for (int i = ttNone; i < ttMaxTrackTypes; i++)
          *availableTracks[i].description = 0;
      }
   else {
-     memset(availableTracks, 0, sizeof(availableTracks));
+     if (IdsOnly) {
+        for (int i = ttNone; i < ttMaxTrackTypes; i++)
+            availableTracks[i].id = 0;
+        }
+     else
+        memset(availableTracks, 0, sizeof(availableTracks));
      pre_1_3_19_PrivateStream = false;
      SetAudioChannel(0); // fall back to stereo
      currentAudioTrackMissingCount = 0;
@@ -794,6 +852,7 @@ int cDevice::NumAudioTracks(void) const
 bool cDevice::SetCurrentAudioTrack(eTrackType Type)
 {
   if (ttNone < Type && Type < ttDolbyLast) {
+     cMutexLock MutexLock(&mutexCurrentAudioTrack);
      if (IS_DOLBY_TRACK(Type))
         SetDigitalAudioDevice(true);
      currentAudioTrack = Type;
@@ -887,15 +946,20 @@ bool cDevice::Replaying(void) const
   return player != NULL;
 }
 
+bool cDevice::Transferring(void) const
+{
+  return dynamic_cast<cTransfer *>(player) != NULL;
+}
+
 bool cDevice::AttachPlayer(cPlayer *Player)
 {
   if (CanReplay()) {
      if (player)
         Detach(player);
-     if (!dynamic_cast<cTransfer *>(Player))
-        ClrAvailableTracks();
      pesAssembler->Reset();
      player = Player;
+     if (!Transferring())
+        ClrAvailableTracks(false, true);
      SetPlayMode(player->playMode);
      player->device = this;
      player->Activate(true);
@@ -940,13 +1004,14 @@ int cDevice::PlayVideo(const uchar *Data, int Length)
   return -1;
 }
 
-int cDevice::PlayAudio(const uchar *Data, int Length)
+int cDevice::PlayAudio(const uchar *Data, int Length, uchar Id)
 {
   return -1;
 }
 
 int cDevice::PlayPesPacket(const uchar *Data, int Length, bool VideoOnly)
 {
+  cMutexLock MutexLock(&mutexCurrentAudioTrack);
   bool FirstLoop = true;
   uchar c = Data[3];
   const uchar *Start = Data;
@@ -962,7 +1027,7 @@ int cDevice::PlayPesPacket(const uchar *Data, int Length, bool VideoOnly)
           case 0xC0 ... 0xDF: // audio
                SetAvailableTrack(ttAudio, c - 0xC0, c);
                if (!VideoOnly && c == availableTracks[currentAudioTrack].id) {
-                  w = PlayAudio(Start, d);
+                  w = PlayAudio(Start, d, c);
                   if (FirstLoop)
                      Audios.PlayAudio(Data, Length, c);
                   }
@@ -972,7 +1037,7 @@ int cDevice::PlayPesPacket(const uchar *Data, int Length, bool VideoOnly)
                uchar SubStreamId = Data[PayloadOffset];
                uchar SubStreamType = SubStreamId & 0xF0;
                uchar SubStreamIndex = SubStreamId & 0x1F;
-        
+
                // Compatibility mode for old VDR recordings, where 0xBD was only AC3:
 pre_1_3_19_PrivateStreamDeteced:
                if (pre_1_3_19_PrivateStream) {
@@ -988,7 +1053,7 @@ pre_1_3_19_PrivateStreamDeteced:
                       if (Setup.UseDolbyDigital) {
                          SetAvailableTrack(ttDolby, SubStreamIndex, SubStreamId);
                          if (!VideoOnly && SubStreamId == availableTracks[currentAudioTrack].id) {
-                            w = PlayAudio(Start, d);
+                            w = PlayAudio(Start, d, SubStreamId);
                             if (FirstLoop)
                                Audios.PlayAudio(Data, Length, SubStreamId);
                             }
@@ -997,7 +1062,7 @@ pre_1_3_19_PrivateStreamDeteced:
                  case 0xA0: // LPCM
                       SetAvailableTrack(ttAudio, SubStreamIndex, SubStreamId);
                       if (!VideoOnly && SubStreamId == availableTracks[currentAudioTrack].id) {
-                         w = PlayAudio(Start, d);
+                         w = PlayAudio(Start, d, SubStreamId);
                          if (FirstLoop)
                             Audios.PlayAudio(Data, Length, SubStreamId);
                          }
@@ -1111,7 +1176,7 @@ int cDevice::ProvidesCa(const cChannel *Channel) const
   int Ca = Channel->Ca();
   if (Ca == CardIndex() + 1)
      return 1; // exactly _this_ card was requested
-  if (Ca && Ca <= MAXDEVICES)
+  if (Ca && Ca <= CA_DVB_MAX)
      return 0; // a specific card was requested, but not _this_ one
   return !Ca; // by default every card can provide FTA
 }
@@ -1242,6 +1307,15 @@ void cDevice::DetachAll(int Pid)
      }
 }
 
+void cDevice::DetachAllReceivers(void)
+{
+  cMutexLock MutexLock(&mutexReceiver);
+  for (int i = 0; i < MAXRECEIVERS; i++) {
+      if (receiver[i])
+         Detach(receiver[i]);
+      }
+}
+
 // --- cTSBuffer -------------------------------------------------------------
 
 cTSBuffer::cTSBuffer(int File, int Size, int CardIndex)
