@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbosd.c 1.27 2005/05/22 10:57:45 kls Exp $
+ * $Id: dvbosd.c 1.30 2006/01/28 14:24:04 kls Exp $
  */
 
 #include "dvbosd.h"
@@ -39,7 +39,7 @@ cDvbOsd::cDvbOsd(int Left, int Top, int OsdDev)
   osdDev = OsdDev;
   shown = false;
   if (osdDev < 0)
-     esyslog("ERROR: illegal OSD device handle (%d)!", osdDev);
+     esyslog("ERROR: invalid OSD device handle (%d)!", osdDev);
   else {
      osdMem = MAXOSDMEMORY;
 #ifdef OSD_CAP_MEMSIZE
@@ -48,13 +48,6 @@ cDvbOsd::cDvbOsd(int Left, int Top, int OsdDev)
      cap.cmd = OSD_CAP_MEMSIZE;
      if (ioctl(osdDev, OSD_GET_CAPABILITY, &cap) == 0)
         osdMem = cap.val;
-     #define FW_NEEDS_BUFFER_RESERVE_FOR_AC3
-     #ifdef FW_NEEDS_BUFFER_RESERVE_FOR_AC3
-     //XXX This is a very ugly hack to allow cDvbOsd to reduce the buffer
-     //XXX requirements in cTransfer if it detects a 4MB full featured DVB card.
-     extern bool DvbCardWith4MBofSDRAM;
-     DvbCardWith4MBofSDRAM = osdMem >= 1000000;
-     #endif
 #endif
      // must clear all windows here to avoid flashing effects - doesn't work if done
      // in Flush() only for the windows that are actually used...
@@ -88,6 +81,8 @@ eOsdError cDvbOsd::CanHandleAreas(const tArea *Areas, int NumAreas)
             return oeBppNotSupported;
          if ((Areas[i].Width() & (8 / Areas[i].bpp - 1)) != 0)
             return oeWrongAlignment;
+         if (Areas[i].Width() < 1 || Areas[i].Height() < 1 || Areas[i].Width() > 720 || Areas[i].Height() > 576)
+            return oeWrongAreaSize;
          TotalMemory += Areas[i].Width() * Areas[i].Height() / (8 / Areas[i].bpp);
          }
      if (TotalMemory > osdMem)
