@@ -8,7 +8,7 @@
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  * Adapted to 'libsi' for VDR 1.3.0 by Marcel Wiesweg <marcel.wiesweg@gmx.de>.
  *
- * $Id: eit.c 1.112 2005/11/04 14:19:16 kls Exp $
+ * $Id: eit.c 1.117 2006/04/29 11:38:37 kls Exp $
  */
 
 #include "eit.h"
@@ -35,21 +35,24 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data)
   if (!channel)
      return; // only collect data for known channels
 
-  cSchedule *pSchedule = (cSchedule *)Schedules->GetSchedule(channelID);
-  if (!pSchedule) {
-     pSchedule = new cSchedule(channelID);
-     Schedules->Add(pSchedule);
-     }
+  cSchedule *pSchedule = (cSchedule *)Schedules->GetSchedule(channel, true);
 
   bool Empty = true;
   bool Modified = false;
+  bool HasExternalData = false;
+  time_t SegmentStart = 0;
+  time_t SegmentEnd = 0;
 
   SI::EIT::Event SiEitEvent;
   for (SI::Loop::Iterator it; eventLoop.getNext(SiEitEvent, it); ) {
+      bool ExternalData = false;
       // Drop bogus events - but keep NVOD reference events, where all bits of the start time field are set to 1, resulting in a negative number.
       if (SiEitEvent.getStartTime() == 0 || SiEitEvent.getStartTime() > 0 && SiEitEvent.getDuration() == 0)
          continue;
       Empty = false;
+      if (!SegmentStart)
+         SegmentStart = SiEitEvent.getStartTime();
+      SegmentEnd = SiEitEvent.getStartTime() + SiEitEvent.getDuration();
       cEvent *newEvent = NULL;
       cEvent *rEvent = NULL;
       cEvent *pEvent = (cEvent *)pSchedule->GetEvent(SiEitEvent.getEventId(), SiEitEvent.getStartTime());
@@ -65,11 +68,14 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data)
          pEvent->SetSeen();
          // If the existing event has a zero table ID it was defined externally and shall
          // not be overwritten.
-         if (pEvent->TableID() == 0x00)
-            continue;
+         if (pEvent->TableID() == 0x00) {
+            if (pEvent->Version() == getVersionNumber())
+               continue;
+            HasExternalData = ExternalData = true;
+            }
          // If the new event has a higher table ID, let's skip it.
          // The lower the table ID, the more "current" the information.
-         if (Tid > pEvent->TableID())
+         else if (Tid > pEvent->TableID())
             continue;
          // If the new event comes from the same table and has the same version number
          // as the existing one, let's skip it to avoid unnecessary work.
@@ -77,15 +83,16 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data)
          // the actual Premiere transponder and the Sat.1/Pro7 transponder), but use different version numbers on
          // each of them :-( So if one DVB card is tuned to the Premiere transponder, while an other one is tuned
          // to the Sat.1/Pro7 transponder, events will keep toggling because of the bogus version numbers.
-         if (Tid == pEvent->TableID() && pEvent->Version() == getVersionNumber())
+         else if (Tid == pEvent->TableID() && pEvent->Version() == getVersionNumber())
             continue;
          }
-      // XXX TODO log different (non-zero) event IDs for the same event???
-      pEvent->SetEventID(SiEitEvent.getEventId()); // unfortunately some stations use different event ids for the same event in different tables :-(
-      pEvent->SetTableID(Tid);
+      if (!ExternalData) {
+         pEvent->SetEventID(SiEitEvent.getEventId()); // unfortunately some stations use different event ids for the same event in different tables :-(
+         pEvent->SetTableID(Tid);
+         pEvent->SetStartTime(SiEitEvent.getStartTime());
+         pEvent->SetDuration(SiEitEvent.getDuration());
+         }
       pEvent->SetVersion(getVersionNumber());
-      pEvent->SetStartTime(SiEitEvent.getStartTime());
-      pEvent->SetDuration(SiEitEvent.getDuration());
 
       int LanguagePreferenceShort = -1;
       int LanguagePreferenceExt = -1;
@@ -96,6 +103,10 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data)
       cLinkChannels *LinkChannels = NULL;
       cComponents *Components = NULL;
       for (SI::Loop::Iterator it2; (d = SiEitEvent.eventDescriptors.getNext(it2)); ) {
+          if (ExternalData && d->getDescriptorTag() != SI::ComponentDescriptorTag) {
+             delete d;
+             continue;
+             }
           switch (d->getDescriptorTag()) {
             case SI::ExtendedEventDescriptorTag: {
                  SI::ExtendedEventDescriptor *eed = (SI::ExtendedEventDescriptor *)d;
@@ -171,10 +182,10 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data)
                        if (link != channel) { // only link to other channels, not the same one
                           //fprintf(stderr, "Linkage %s %4d %4d %5d %5d %5d %5d  %02X  '%s'\n", hit ? "*" : "", channel->Number(), link ? link->Number() : -1, SiEitEvent.getEventId(), ld->getOriginalNetworkId(), ld->getTransportStreamId(), ld->getServiceId(), ld->getLinkageType(), linkName);//XXX
                           if (link) {
-                             if (Setup.UpdateChannels >= 1)
+                             if (Setup.UpdateChannels == 1 || Setup.UpdateChannels >= 3)
                                 link->SetName(linkName, "", "");
                              }
-                          else if (Setup.UpdateChannels >= 3) {
+                          else if (Setup.UpdateChannels >= 4) {
                              link = Channels.NewChannel(channel, linkName, "", "", ld->getOriginalNetworkId(), ld->getTransportStreamId(), ld->getServiceId());
                              //XXX patFilter->Trigger();
                              }
@@ -242,6 +253,8 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data)
      pSchedule->SetPresentSeen();
   if (Modified) {
      pSchedule->Sort();
+     if (!HasExternalData)
+        pSchedule->DropOutdated(SegmentStart, SegmentEnd, Tid, getVersionNumber());
      Schedules->SetModified(pSchedule);
      }
 }
