@@ -7,28 +7,30 @@
  * Original version (as used in VDR before 1.3.0) written by
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  *
- * $Id: epg.c 1.40 2005/11/11 13:37:43 kls Exp $
+ * $Id: epg.c 1.72 2006/04/22 12:02:47 kls Exp $
  */
 
 #include "epg.h"
-#include "libsi/si.h"
-#include "timers.h"
 #include <ctype.h>
 #include <time.h>
+#include "libsi/si.h"
+#include "timers.h"
+
+#define RUNNINGSTATUSTIMEOUT 30 // seconds before the running status is considered unknown
 
 // --- tComponent ------------------------------------------------------------
 
 cString tComponent::ToString(void)
 {
   char buffer[256];
-  snprintf(buffer, sizeof(buffer), "%X %02X %-3s %s", stream, type, language, description ? description : "");
+  snprintf(buffer, sizeof(buffer), "%X %02X %s %s", stream, type, language, description ? description : "");
   return buffer;
 }
 
 bool tComponent::FromString(const char *s)
 {
   unsigned int Stream, Type;
-  int n = sscanf(s, "%X %02X %3c %a[^\n]", &Stream, &Type, language, &description);
+  int n = sscanf(s, "%X %02X %7s %a[^\n]", &Stream, &Type, language, &description); // 7 = MAXLANGCODE2 - 1
   if (n != 4 || isempty(description)) {
      free(description);
      description = NULL;
@@ -76,18 +78,32 @@ void cComponents::SetComponent(int Index, uchar Stream, uchar Type, const char *
   p->stream = Stream;
   p->type = Type;
   strn0cpy(p->language, Language, sizeof(p->language));
+  char *q = strchr(p->language, ',');
+  if (q)
+     *q = 0; // strips rest of "normalized" language codes
   p->description = strcpyrealloc(p->description, !isempty(Description) ? Description : NULL);
 }
 
+tComponent *cComponents::GetComponent(int Index, uchar Stream, uchar Type)
+{
+  for (int i = 0; i < numComponents; i++) {
+      if (components[i].stream == Stream && components[i].type == Type) {
+         if (!Index--)
+            return &components[i];
+         }
+      }
+  return NULL;
+}
+
 // --- cEvent ----------------------------------------------------------------
 
-cEvent::cEvent(u_int16_t EventID)
+cEvent::cEvent(tEventID EventID)
 {
   schedule = NULL;
   eventID = EventID;
   tableID = 0;
   version = 0xFF; // actual version numbers are 0..31
-  runningStatus = 0;
+  runningStatus = SI::RunningStatusUndefined;
   title = NULL;
   shortText = NULL;
   description = NULL;
@@ -117,7 +133,7 @@ tChannelID cEvent::ChannelID(void) const
   return schedule ? schedule->ChannelID() : tChannelID();
 }
 
-void cEvent::SetEventID(u_int16_t EventID)
+void cEvent::SetEventID(tEventID EventID)
 {
   if (eventID != EventID) {
      if (schedule)
@@ -140,9 +156,8 @@ void cEvent::SetVersion(uchar Version)
 
 void cEvent::SetRunningStatus(int RunningStatus, cChannel *Channel)
 {
-  if (Channel && runningStatus != RunningStatus && (RunningStatus > SI::RunningStatusNotRunning || runningStatus > SI::RunningStatusUndefined))
-     if (Channel->Number() <= 30)//XXX maybe log only those that have timers???
-     isyslog("channel %d (%s) event %s '%s' status %d", Channel->Number(), Channel->Name(), *GetTimeString(), Title(), RunningStatus);
+  if (Channel && runningStatus != RunningStatus && (RunningStatus > SI::RunningStatusNotRunning || runningStatus > SI::RunningStatusUndefined) && Channel->HasTimer())
+     isyslog("channel %d (%s) event %s status %d", Channel->Number(), Channel->Name(), *ToDescr(), RunningStatus);
   runningStatus = RunningStatus;
 }
 
@@ -193,6 +208,14 @@ void cEvent::SetSeen(void)
   seen = time(NULL);
 }
 
+cString cEvent::ToDescr(void) const
+{
+  char vpsbuf[64] = "";
+  if (Vps())
+     sprintf(vpsbuf, "(VPS: %s) ", *GetVpsString());
+  return cString::sprintf("%s %s-%s %s'%s'", *GetDateString(), *GetTimeString(), *GetEndTimeString(), vpsbuf, Title());
+}
+
 bool cEvent::HasTimer(void) const
 {
   for (cTimer *t = Timers.First(); t; t = Timers.Next(t)) {
@@ -204,7 +227,7 @@ bool cEvent::HasTimer(void) const
 
 bool cEvent::IsRunning(bool OrAboutToStart) const
 {
-  return runningStatus >= (OrAboutToStart ? SI::RunningStatusStartsInAFewSeconds : SI::RunningStatusPausing);
+  return SeenWithin(RUNNINGSTATUSTIMEOUT) && runningStatus >= (OrAboutToStart ? SI::RunningStatusStartsInAFewSeconds : SI::RunningStatusPausing);
 }
 
 cString cEvent::GetDateString(void) const
@@ -233,8 +256,7 @@ cString cEvent::GetVpsString(void) const
 void cEvent::Dump(FILE *f, const char *Prefix, bool InfoOnly) const
 {
   if (InfoOnly || startTime + duration + Setup.EPGLinger * 60 >= time(NULL)) {
-     if (!InfoOnly)
-        fprintf(f, "%sE %u %ld %d %X\n", Prefix, eventID, startTime, duration, tableID);
+     fprintf(f, "%sE %u %ld %d %X %X\n", Prefix, eventID, startTime, duration, tableID, version);
      if (!isempty(title))
         fprintf(f, "%sT %s\n", Prefix, title);
      if (!isempty(shortText))
@@ -247,10 +269,12 @@ void cEvent::Dump(FILE *f, const char *Prefix, bool InfoOnly) const
      if (components) {
         for (int i = 0; i < components->NumComponents(); i++) {
             tComponent *p = components->Component(i);
+            if (!Setup.UseDolbyDigital && p->stream == 0x02 && p->type == 0x05)
+               continue;
             fprintf(f, "%sX %s\n", Prefix, *p->ToString());
             }
         }
-     if (!InfoOnly && vps)
+     if (vps)
         fprintf(f, "%sV %ld\n", Prefix, vps);
      if (!InfoOnly)
         fprintf(f, "%se\n", Prefix);
@@ -296,8 +320,9 @@ bool cEvent::Read(FILE *f, cSchedule *Schedule)
                           time_t StartTime;
                           int Duration;
                           unsigned int TableID = 0;
-                          int n = sscanf(t, "%u %ld %d %X", &EventID, &StartTime, &Duration, &TableID);
-                          if (n == 3 || n == 4) {
+                          unsigned int Version = 0xFF; // actual value is ignored
+                          int n = sscanf(t, "%u %ld %d %X %X", &EventID, &StartTime, &Duration, &TableID, &Version);
+                          if (n >= 3 && n <= 5) {
                              Event = (cEvent *)Schedule->GetEvent(EventID, StartTime);
                              cEvent *newEvent = NULL;
                              if (Event)
@@ -314,7 +339,9 @@ bool cEvent::Read(FILE *f, cSchedule *Schedule)
                              }
                           }
                        break;
-             case 'e': Event = NULL;
+             case 'e': if (Event && !Event->Title())
+                          Event->SetTitle(tr("No title"));
+                       Event = NULL;
                        break;
              case 'c': // to keep things simple we react on 'c' here
                        return true;
@@ -418,197 +445,196 @@ void cEvent::FixEpgBugs(void)
   strreplace(description, '\x86', ' ');
   strreplace(description, '\x87', ' ');
 
+  if (isempty(title)) {
+     // we don't want any "(null)" titles
+     title = strcpyrealloc(title, tr("No title"));
+     EpgBugFixStat(12, ChannelID());
+     }
+
   if (Setup.EPGBugfixLevel == 0)
      return;
 
   // Some TV stations apparently have their own idea about how to fill in the
   // EPG data. Let's fix their bugs as good as we can:
-  if (title) {
 
-     // Some channels put the ShortText in quotes and use either the ShortText
-     // or the Description field, depending on how long the string is:
-     //
-     // Title
-     // "ShortText". Description
-     //
-     if ((shortText == NULL) != (description == NULL)) {
-        char *p = shortText ? shortText : description;
-        if (*p == '"') {
-           const char *delim = "\".";
-           char *e = strstr(p + 1, delim);
-           if (e) {
-              *e = 0;
-              char *s = strdup(p + 1);
-              char *d = strdup(e + strlen(delim));
-              free(shortText);
-              free(description);
-              shortText = s;
-              description = d;
-              EpgBugFixStat(1, ChannelID());
-              }
-           }
-        }
-
-     // Some channels put the Description into the ShortText (preceded
-     // by a blank) if there is no actual ShortText and the Description
-     // is short enough:
-     //
-     // Title
-     //  Description
-     //
-     if (shortText && !description) {
-        if (*shortText == ' ') {
-           memmove(shortText, shortText + 1, strlen(shortText));
-           description = shortText;
-           shortText = NULL;
-           EpgBugFixStat(2, ChannelID());
-           }
-        }
-
-     // Sometimes they repeat the Title in the ShortText:
-     //
-     // Title
-     // Title
-     //
-     if (shortText && strcmp(title, shortText) == 0) {
-        free(shortText);
-        shortText = NULL;
-        EpgBugFixStat(3, ChannelID());
-        }
-
-     // Some channels put the ShortText between double quotes, which is nothing
-     // but annoying (some even put a '.' after the closing '"'):
-     //
-     // Title
-     // "ShortText"[.]
-     //
-     if (shortText && *shortText == '"') {
-        int l = strlen(shortText);
-        if (l > 2 && (shortText[l - 1] == '"' || (shortText[l - 1] == '.' && shortText[l - 2] == '"'))) {
-           memmove(shortText, shortText + 1, l);
-           char *p = strrchr(shortText, '"');
-           if (p)
-              *p = 0;
-           EpgBugFixStat(4, ChannelID());
-           }
-        }
-
-     if (Setup.EPGBugfixLevel <= 1)
-        return;
-
-     // Some channels apparently try to do some formatting in the texts,
-     // which is a bad idea because they have no way of knowing the width
-     // of the window that will actually display the text.
-     // Remove excess whitespace:
-     title = compactspace(title);
-     shortText = compactspace(shortText);
-     description = compactspace(description);
-
-#define MAX_USEFUL_EPISODE_LENGTH 40
-     // Some channels put a whole lot of information in the ShortText and leave
-     // the Description totally empty. So if the ShortText length exceeds
-     // MAX_USEFUL_EPISODE_LENGTH, let's put this into the Description
-     // instead:
-     if (!isempty(shortText) && isempty(description)) {
-        if (strlen(shortText) > MAX_USEFUL_EPISODE_LENGTH) {
-           free(description);
-           description = shortText;
-           shortText = NULL;
-           EpgBugFixStat(6, ChannelID());
-           }
-        }
-
-     // Some channels put the same information into ShortText and Description.
-     // In that case we delete one of them:
-     if (shortText && description && strcmp(shortText, description) == 0) {
-        if (strlen(shortText) > MAX_USEFUL_EPISODE_LENGTH) {
+  // Some channels put the ShortText in quotes and use either the ShortText
+  // or the Description field, depending on how long the string is:
+  //
+  // Title
+  // "ShortText". Description
+  //
+  if ((shortText == NULL) != (description == NULL)) {
+     char *p = shortText ? shortText : description;
+     if (*p == '"') {
+        const char *delim = "\".";
+        char *e = strstr(p + 1, delim);
+        if (e) {
+           *e = 0;
+           char *s = strdup(p + 1);
+           char *d = strdup(e + strlen(delim));
            free(shortText);
-           shortText = NULL;
-           }
-        else {
            free(description);
-           description = NULL;
+           shortText = s;
+           description = d;
+           EpgBugFixStat(1, ChannelID());
            }
-        EpgBugFixStat(7, ChannelID());
-        }
-
-     // Some channels use the ` ("backtick") character, where a ' (single quote)
-     // would be normally used. Actually, "backticks" in normal text don't make
-     // much sense, so let's replace them:
-     strreplace(title, '`', '\'');
-     strreplace(shortText, '`', '\'');
-     strreplace(description, '`', '\'');
-
-     if (Setup.EPGBugfixLevel <= 2)
-        return;
-
-     // The stream components have a "description" field which some channels
-     // apparently have no idea of how to set correctly:
-     if (components) {
-        for (int i = 0; i < components->NumComponents(); i++) {
-            tComponent *p = components->Component(i);
-            switch (p->stream) {
-              case 0x01: { // video
-                   if (p->description) {
-                      if (strcasecmp(p->description, "Video") == 0 ||
-                           strcasecmp(p->description, "Bildformat") == 0) {
-                         // Yes, we know it's video - that's what the 'stream' code
-                         // is for! But _which_ video is it?
-                         free(p->description);
-                         p->description = NULL;
-                         EpgBugFixStat(8, ChannelID());
-                         }
-                      }
-                   if (!p->description) {
-                      switch (p->type) {
-                        case 0x01:
-                        case 0x05: p->description = strdup("4:3"); break;
-                        case 0x02:
-                        case 0x03:
-                        case 0x06:
-                        case 0x07: p->description = strdup("16:9"); break;
-                        case 0x04:
-                        case 0x08: p->description = strdup(">16:9"); break;
-                        case 0x09:
-                        case 0x0D: p->description = strdup("HD 4:3"); break;
-                        case 0x0A:
-                        case 0x0B:
-                        case 0x0E:
-                        case 0x0F: p->description = strdup("HD 16:9"); break;
-                        case 0x0C:
-                        case 0x10: p->description = strdup("HD >16:9"); break;
-                        }
-                      EpgBugFixStat(9, ChannelID());
-                      }
-                   }
-                   break;
-              case 0x02: { // audio
-                   if (p->description) {
-                      if (strcasecmp(p->description, "Audio") == 0) {
-                         // Yes, we know it's audio - that's what the 'stream' code
-                         // is for! But _which_ audio is it?
-                         free(p->description);
-                         p->description = NULL;
-                         EpgBugFixStat(10, ChannelID());
-                         }
-                      }
-                   if (!p->description) {
-                      switch (p->type) {
-                        case 0x05: p->description = strdup("Dolby Digital"); break;
-                        // all others will just display the language
-                        }
-                      EpgBugFixStat(11, ChannelID());
-                      }
-                   }
-                   break;
-              }
-            }
         }
      }
-  else {
-     // we don't want any "(null)" titles
-     title = strcpyrealloc(title, tr("No title"));
-     EpgBugFixStat(12, ChannelID());
+
+  // Some channels put the Description into the ShortText (preceded
+  // by a blank) if there is no actual ShortText and the Description
+  // is short enough:
+  //
+  // Title
+  //  Description
+  //
+  if (shortText && !description) {
+     if (*shortText == ' ') {
+        memmove(shortText, shortText + 1, strlen(shortText));
+        description = shortText;
+        shortText = NULL;
+        EpgBugFixStat(2, ChannelID());
+        }
+     }
+
+  // Sometimes they repeat the Title in the ShortText:
+  //
+  // Title
+  // Title
+  //
+  if (shortText && strcmp(title, shortText) == 0) {
+     free(shortText);
+     shortText = NULL;
+     EpgBugFixStat(3, ChannelID());
+     }
+
+  // Some channels put the ShortText between double quotes, which is nothing
+  // but annoying (some even put a '.' after the closing '"'):
+  //
+  // Title
+  // "ShortText"[.]
+  //
+  if (shortText && *shortText == '"') {
+     int l = strlen(shortText);
+     if (l > 2 && (shortText[l - 1] == '"' || (shortText[l - 1] == '.' && shortText[l - 2] == '"'))) {
+        memmove(shortText, shortText + 1, l);
+        char *p = strrchr(shortText, '"');
+        if (p)
+           *p = 0;
+        EpgBugFixStat(4, ChannelID());
+        }
+     }
+
+  if (Setup.EPGBugfixLevel <= 1)
+     return;
+
+  // Some channels apparently try to do some formatting in the texts,
+  // which is a bad idea because they have no way of knowing the width
+  // of the window that will actually display the text.
+  // Remove excess whitespace:
+  title = compactspace(title);
+  shortText = compactspace(shortText);
+  description = compactspace(description);
+
+#define MAX_USEFUL_EPISODE_LENGTH 40
+  // Some channels put a whole lot of information in the ShortText and leave
+  // the Description totally empty. So if the ShortText length exceeds
+  // MAX_USEFUL_EPISODE_LENGTH, let's put this into the Description
+  // instead:
+  if (!isempty(shortText) && isempty(description)) {
+     if (strlen(shortText) > MAX_USEFUL_EPISODE_LENGTH) {
+        free(description);
+        description = shortText;
+        shortText = NULL;
+        EpgBugFixStat(6, ChannelID());
+        }
+     }
+
+  // Some channels put the same information into ShortText and Description.
+  // In that case we delete one of them:
+  if (shortText && description && strcmp(shortText, description) == 0) {
+     if (strlen(shortText) > MAX_USEFUL_EPISODE_LENGTH) {
+        free(shortText);
+        shortText = NULL;
+        }
+     else {
+        free(description);
+        description = NULL;
+        }
+     EpgBugFixStat(7, ChannelID());
+     }
+
+  // Some channels use the ` ("backtick") character, where a ' (single quote)
+  // would be normally used. Actually, "backticks" in normal text don't make
+  // much sense, so let's replace them:
+  strreplace(title, '`', '\'');
+  strreplace(shortText, '`', '\'');
+  strreplace(description, '`', '\'');
+
+  if (Setup.EPGBugfixLevel <= 2)
+     return;
+
+  // The stream components have a "description" field which some channels
+  // apparently have no idea of how to set correctly:
+  if (components) {
+     for (int i = 0; i < components->NumComponents(); i++) {
+         tComponent *p = components->Component(i);
+         switch (p->stream) {
+           case 0x01: { // video
+                if (p->description) {
+                   if (strcasecmp(p->description, "Video") == 0 ||
+                        strcasecmp(p->description, "Bildformat") == 0) {
+                      // Yes, we know it's video - that's what the 'stream' code
+                      // is for! But _which_ video is it?
+                      free(p->description);
+                      p->description = NULL;
+                      EpgBugFixStat(8, ChannelID());
+                      }
+                   }
+                if (!p->description) {
+                   switch (p->type) {
+                     case 0x01:
+                     case 0x05: p->description = strdup("4:3"); break;
+                     case 0x02:
+                     case 0x03:
+                     case 0x06:
+                     case 0x07: p->description = strdup("16:9"); break;
+                     case 0x04:
+                     case 0x08: p->description = strdup(">16:9"); break;
+                     case 0x09:
+                     case 0x0D: p->description = strdup("HD 4:3"); break;
+                     case 0x0A:
+                     case 0x0B:
+                     case 0x0E:
+                     case 0x0F: p->description = strdup("HD 16:9"); break;
+                     case 0x0C:
+                     case 0x10: p->description = strdup("HD >16:9"); break;
+                     }
+                   EpgBugFixStat(9, ChannelID());
+                   }
+                }
+                break;
+           case 0x02: { // audio
+                if (p->description) {
+                   if (strcasecmp(p->description, "Audio") == 0) {
+                      // Yes, we know it's audio - that's what the 'stream' code
+                      // is for! But _which_ audio is it?
+                      free(p->description);
+                      p->description = NULL;
+                      EpgBugFixStat(10, ChannelID());
+                      }
+                   }
+                if (!p->description) {
+                   switch (p->type) {
+                     case 0x05: p->description = strdup("Dolby Digital"); break;
+                     // all others will just display the language
+                     }
+                   EpgBugFixStat(11, ChannelID());
+                   }
+                }
+                break;
+           }
+         }
      }
 }
 
@@ -635,7 +661,6 @@ void cSchedule::DelEvent(cEvent *Event)
   if (Event->schedule == this) {
      UnhashEvent(Event);
      events.Del(Event);
-     Event->schedule = NULL;
      }
 }
 
@@ -653,31 +678,37 @@ void cSchedule::UnhashEvent(cEvent *Event)
      eventsHashStartTime.Del(Event, Event->StartTime());
 }
 
-const cEvent *cSchedule::GetPresentEvent(bool CheckRunningStatus) const
+const cEvent *cSchedule::GetPresentEvent(void) const
 {
   const cEvent *pe = NULL;
   time_t now = time(NULL);
   for (cEvent *p = events.First(); p; p = events.Next(p)) {
-      if (p->StartTime() <= now && now < p->EndTime()) {
+      if (p->StartTime() <= now)
          pe = p;
-         if (!CheckRunningStatus)
-            break;
-         }
-      if (CheckRunningStatus && p->SeenWithin(30) && p->RunningStatus() >= SI::RunningStatusPausing)
+      else if (p->StartTime() > now + 3600)
+         break;
+      if (p->SeenWithin(RUNNINGSTATUSTIMEOUT) && p->RunningStatus() >= SI::RunningStatusPausing)
          return p;
       }
   return pe;
 }
 
-const cEvent *cSchedule::GetFollowingEvent(bool CheckRunningStatus) const
+const cEvent *cSchedule::GetFollowingEvent(void) const
 {
-  const cEvent *p = GetPresentEvent(CheckRunningStatus);
+  const cEvent *p = GetPresentEvent();
   if (p)
      p = events.Next(p);
+  else {
+     time_t now = time(NULL);
+     for (p = events.First(); p; p = events.Next(p)) {
+         if (p->StartTime() >= now)
+            break;
+         }
+     }
   return p;
 }
 
-const cEvent *cSchedule::GetEvent(u_int16_t EventID, time_t StartTime) const
+const cEvent *cSchedule::GetEvent(tEventID EventID, time_t StartTime) const
 {
   // Returns either the event info with the given EventID or, if that one can't
   // be found, the one with the given StartTime (or NULL if neither can be found)
@@ -704,9 +735,11 @@ const cEvent *cSchedule::GetEventAround(time_t Time) const
 void cSchedule::SetRunningStatus(cEvent *Event, int RunningStatus, cChannel *Channel)
 {
   for (cEvent *p = events.First(); p; p = events.Next(p)) {
-      if (p == Event)
-         p->SetRunningStatus(RunningStatus, Channel);
-      else if (RunningStatus >= SI::RunningStatusPausing && p->RunningStatus() > SI::RunningStatusNotRunning)
+      if (p == Event) {
+         if (p->RunningStatus() > SI::RunningStatusNotRunning || RunningStatus > SI::RunningStatusNotRunning)
+            p->SetRunningStatus(RunningStatus, Channel);
+         }
+      else if (RunningStatus >= SI::RunningStatusPausing && p->StartTime() < Event->StartTime())
          p->SetRunningStatus(SI::RunningStatusNotRunning);
       }
   if (RunningStatus >= SI::RunningStatusPausing)
@@ -737,6 +770,31 @@ void cSchedule::Sort(void)
   events.Sort();
 }
 
+void cSchedule::DropOutdated(time_t SegmentStart, time_t SegmentEnd, uchar TableID, uchar Version)
+{
+  if (SegmentStart > 0 && SegmentEnd > 0) {
+     for (cEvent *p = events.First(); p; p = events.Next(p)) {
+         if (p->EndTime() > SegmentStart) {
+            if (p->StartTime() < SegmentEnd) {
+               // The event overlaps with the given time segment.
+               if (p->TableID() > TableID || p->TableID() == TableID && p->Version() != Version) {
+                  // The segment overwrites all events from tables with higher ids, and
+                  // within the same table id all events must have the same version.
+                  // We can't delete the event right here because a timer might have
+                  // a pointer to it, so let's set its id and start time to 0 to have it
+                  // "phased out":
+                  UnhashEvent(p);
+                  p->eventID = 0;
+                  p->startTime = 0;
+                  }
+               }
+            else
+               break;
+            }
+         }
+     }
+}
+
 void cSchedule::Cleanup(void)
 {
   Cleanup(time(NULL));
@@ -745,15 +803,12 @@ void cSchedule::Cleanup(void)
 void cSchedule::Cleanup(time_t Time)
 {
   cEvent *Event;
-  for (int a = 0; true ; a++) {
-      Event = events.Get(a);
-      if (!Event)
-         break;
-      if (!Event->HasTimer() && Event->EndTime() + Setup.EPGLinger * 60 + 3600 < Time) { // adding one hour for safety
-         DelEvent(Event);
-         a--;
-         }
-      }
+  while ((Event = events.First()) != NULL) {
+        if (!Event->HasTimer() && Event->EndTime() + Setup.EPGLinger * 60 + 3600 < Time) // adding one hour for safety
+           DelEvent(Event);
+        else
+           break;
+        }
 }
 
 void cSchedule::Dump(FILE *f, const char *Prefix, eDumpMode DumpMode, time_t AtTime) const
@@ -811,7 +866,7 @@ bool cSchedule::Read(FILE *f, cSchedules *Schedules)
                        }
                     }
                  else {
-                    esyslog("ERROR: illegal channel ID: %s", s);
+                    esyslog("ERROR: invalid channel ID: %s", s);
                     return false;
                     }
                  }
@@ -871,7 +926,7 @@ void cSchedules::Cleanup(bool Force)
   time_t now = time(NULL);
   struct tm tm_r;
   struct tm *ptm = localtime_r(&now, &tm_r);
-  if (now - lastCleanup > 3600 && ptm->tm_hour == 5) {
+  if (now - lastCleanup > 3600) {
      isyslog("cleaning up schedules data");
      cSchedulesLock SchedulesLock(true, 1000);
      cSchedules *s = (cSchedules *)Schedules(SchedulesLock);
@@ -880,7 +935,8 @@ void cSchedules::Cleanup(bool Force)
             p->Cleanup(now);
         }
      lastCleanup = now;
-     ReportEpgBugFixStats(true);
+     if (ptm->tm_hour == 5)
+        ReportEpgBugFixStats(true);
      }
   if (epgDataFileName && now - lastDump > 600) {
      cSafeFile f(epgDataFileName);
@@ -909,7 +965,10 @@ bool cSchedules::ClearAll(void)
   cSchedulesLock SchedulesLock(true, 1000);
   cSchedules *s = (cSchedules *)Schedules(SchedulesLock);
   if (s) {
-     s->Clear();
+     for (cTimer *Timer = Timers.First(); Timer; Timer = Timers.Next(Timer))
+         Timer->SetEvent(NULL);
+     for (cSchedule *Schedule = s->First(); Schedule; Schedule = s->Next(Schedule))
+         Schedule->Cleanup(INT_MAX);
      return true;
      }
   return false;
@@ -947,6 +1006,11 @@ bool cSchedules::Read(FILE *f)
      bool result = cSchedule::Read(f, s);
      if (OwnFile)
         fclose(f);
+     if (result) {
+        // Initialize the channels' schedule pointers, so that the first WhatsOn menu will come up faster:
+        for (cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel))
+            s->GetSchedule(Channel);
+        }
      return result;
      }
   return false;
@@ -973,3 +1037,19 @@ const cSchedule *cSchedules::GetSchedule(tChannelID ChannelID) const
   return NULL;
 }
 
+const cSchedule *cSchedules::GetSchedule(const cChannel *Channel, bool AddIfMissing) const
+{
+  // This is not very beautiful, but it dramatically speeds up the
+  // "What's on now/next?" menus.
+  static cSchedule DummySchedule(tChannelID::InvalidID);
+  if (!Channel->schedule)
+     Channel->schedule = GetSchedule(Channel->GetChannelID());
+  if (!Channel->schedule)
+     Channel->schedule = &DummySchedule;
+  if (Channel->schedule == &DummySchedule && AddIfMissing) {
+     cSchedule *Schedule = new cSchedule(Channel->GetChannelID());
+     ((cSchedules *)this)->Add(Schedule);
+     Channel->schedule = Schedule;
+     }
+  return Channel->schedule != &DummySchedule? Channel->schedule : NULL;
+}
