@@ -7,7 +7,7 @@
  * Original version (as used in VDR before 1.3.0) written by
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  *
- * $Id: epg.h 1.26 2005/09/11 12:54:30 kls Exp $
+ * $Id: epg.h 1.34 2006/03/25 12:39:39 kls Exp $
  */
 
 #ifndef __EPG_H
@@ -24,7 +24,7 @@ enum eDumpMode { dmAll, dmPresent, dmFollowing, dmAtTime };
 struct tComponent {
   uchar stream;
   uchar type;
-  char language[4];
+  char language[MAXLANGCODE2];
   char *description;
   cString ToString(void);
   bool FromString(const char *s);
@@ -42,15 +42,18 @@ public:
   void SetComponent(int Index, const char *s);
   void SetComponent(int Index, uchar Stream, uchar Type, const char *Language, const char *Description);
   tComponent *Component(int Index) const { return (Index < numComponents) ? &components[Index] : NULL; }
+  tComponent *GetComponent(int Index, uchar Stream, uchar Type); // Gets the Index'th component of Stream and Type, skipping other components
   };
 
 class cSchedule;
 
+typedef u_int32_t tEventID;
+
 class cEvent : public cListObject {
   friend class cSchedule;
 private:
   cSchedule *schedule;     // The Schedule this event belongs to
-  u_int16_t eventID;       // Event ID of this event
+  tEventID eventID;        // Event ID of this event
   uchar tableID;           // Table ID this event came from
   uchar version;           // Version number of section this event came from
   int runningStatus;       // 0=undefined, 1=not running, 2=starts in a few seconds, 3=pausing, 4=running
@@ -63,11 +66,12 @@ private:
   time_t vps;              // Video Programming Service timestamp (VPS, aka "Programme Identification Label", PIL)
   time_t seen;             // When this event was last seen in the data stream
 public:
-  cEvent(u_int16_t EventID);
+  cEvent(tEventID EventID);
   ~cEvent();
   virtual int Compare(const cListObject &ListObject) const;
   tChannelID ChannelID(void) const;
-  u_int16_t EventID(void) const { return eventID; }
+  const cSchedule *Schedule(void) const { return schedule; }
+  tEventID EventID(void) const { return eventID; }
   uchar TableID(void) const { return tableID; }
   uchar Version(void) const { return version; }
   int RunningStatus(void) const { return runningStatus; }
@@ -87,7 +91,7 @@ public:
   cString GetTimeString(void) const;
   cString GetEndTimeString(void) const;
   cString GetVpsString(void) const;
-  void SetEventID(u_int16_t EventID);
+  void SetEventID(tEventID EventID);
   void SetTableID(uchar TableID);
   void SetVersion(uchar Version);
   void SetRunningStatus(int RunningStatus, cChannel *Channel = NULL);
@@ -99,6 +103,7 @@ public:
   void SetDuration(int Duration);
   void SetVps(time_t Vps);
   void SetSeen(void);
+  cString ToDescr(void) const;
   void Dump(FILE *f, const char *Prefix = "", bool InfoOnly = false) const;
   bool Parse(char *s);
   static bool Read(FILE *f, cSchedule *Schedule);
@@ -128,6 +133,7 @@ public:
   void ClrRunningStatus(cChannel *Channel = NULL);
   void ResetVersions(void);
   void Sort(void);
+  void DropOutdated(time_t SegmentStart, time_t SegmentEnd, uchar TableID, uchar Version);
   void Cleanup(time_t Time);
   void Cleanup(void);
   cEvent *AddEvent(cEvent *Event);
@@ -135,9 +141,9 @@ public:
   void HashEvent(cEvent *Event);
   void UnhashEvent(cEvent *Event);
   const cList<cEvent> *Events(void) const { return &events; }
-  const cEvent *GetPresentEvent(bool CheckRunningStatus = false) const;
-  const cEvent *GetFollowingEvent(bool CheckRunningStatus = false) const;
-  const cEvent *GetEvent(u_int16_t EventID, time_t StartTime = 0) const;
+  const cEvent *GetPresentEvent(void) const;
+  const cEvent *GetFollowingEvent(void) const;
+  const cEvent *GetEvent(tEventID EventID, time_t StartTime = 0) const;
   const cEvent *GetEventAround(time_t Time) const;
   void Dump(FILE *f, const char *Prefix = "", eDumpMode DumpMode = dmAll, time_t AtTime = 0) const;
   static bool Read(FILE *f, cSchedules *Schedules);
@@ -177,6 +183,7 @@ public:
   static bool Read(FILE *f = NULL);
   cSchedule *AddSchedule(tChannelID ChannelID);
   const cSchedule *GetSchedule(tChannelID ChannelID) const;
+  const cSchedule *GetSchedule(const cChannel *Channel, bool AddIfMissing = false) const;
   };
 
 void ReportEpgBugFixStats(bool Reset = false);
