@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: keys.c 1.9 2005/09/17 11:27:40 kls Exp $
+ * $Id: keys.c 1.13 2006/04/15 13:50:43 kls Exp $
  */
 
 #include "keys.h"
@@ -32,15 +32,19 @@ static tKey keyTable[] = { // "Up" and "Down" must be the first two keys!
                     { k7,             "7"          },
                     { k8,             "8"          },
                     { k9,             "9"          },
+                    { kInfo,          "Info"       },
                     { kPlay,          "Play"       },
                     { kPause,         "Pause"      },
                     { kStop,          "Stop"       },
                     { kRecord,        "Record"     },
                     { kFastFwd,       "FastFwd"    },
                     { kFastRew,       "FastRew"    },
+                    { kNext,          "Next"       },
+                    { kPrev,          "Prev"       },
                     { kPower,         "Power"      },
                     { kChanUp,        "Channel+"   },
                     { kChanDn,        "Channel-"   },
+                    { kChanPrev,      "PrevChannel"},
                     { kVolUp,         "Volume+"    },
                     { kVolDn,         "Volume-"    },
                     { kMute,          "Mute"       },
@@ -249,9 +253,11 @@ cKeyMacros KeyMacros;
 
 const cKeyMacro *cKeyMacros::Get(eKeys Key)
 {
-  for (cKeyMacro *k = First(); k; k = Next(k)) {
-      if (*k->Macro() == Key)
-         return k;
-      }
+  if (Key != kNone) {
+     for (cKeyMacro *k = First(); k; k = Next(k)) {
+         if (*k->Macro() == Key)
+            return k;
+         }
+     }
   return NULL;
 }
