@@ -6,7 +6,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   $Id: si.h 1.12 2005/09/03 15:19:00 kls Exp $
+ *   $Id: si.h 1.14 2006/04/14 10:53:44 kls Exp $
  *                                                                         *
  ***************************************************************************/
 
@@ -43,7 +43,8 @@ enum TableId { TableIdPAT = 0x00, //program association section
                TableIdTOT = 0x73, //time offset section
                TableIdDIT = 0x7E, //discontinuity information section
                TableIdSIT = 0x7F, //service information section
-               TableIdAIT = 0x74  //application information section
+               TableIdAIT = 0x74, //application information section
+               TableIdPremiereCIT = 0xA0 //premiere content information section
              };
 
 
@@ -139,13 +140,15 @@ enum DescriptorTag {
                MHP_PrefetchDescriptorTag = 0x0C,
                MHP_DelegatedApplicationDescriptorTag = 0x0E,
                MHP_ApplicationStorageDescriptorTag = 0x10,
+  // Premiere private Descriptor Tags
+               PremiereContentTransmissionDescriptorTag = 0xF2,
 
                //a descriptor currently unimplemented in this library
                //the actual value 0xFF is "forbidden" according to the spec.
                UnimplementedDescriptorTag = 0xFF
 };
 
-enum DescriptorTagDomain { SI, MHP };
+enum DescriptorTagDomain { SI, MHP, PCIT };
 
 enum RunningStatus { RunningStatusUndefined = 0,
                      RunningStatusNotRunning = 1,
@@ -184,7 +187,7 @@ public:
    Object();
    Object(CharArray &d);
    //can only be called once since data is immutable
-   void setData(const unsigned char*data, unsigned int size, bool doCopy=true);
+   void setData(const unsigned char*data, int size, bool doCopy=true);
    CharArray getData() { return data; }
    //returns the valid flag which indicates if data is all right or errors have been encountered
    bool isValid() { return data.isValid(); }
@@ -196,7 +199,7 @@ protected:
    void setData(CharArray &d);
    //returns whether the given offset fits within the limits of the actual data
    //The valid flag will be set accordingly
-   bool checkSize(unsigned int offset);
+   bool checkSize(int offset);
 };
 
 class Section : public Object {
@@ -242,7 +245,7 @@ public:
    //never forget to call this
    void setData(CharArray d, int l) { Object::setData(d); checkSize(l); length=l; }
    //convenience method
-   void setDataAndOffset(CharArray d, int l, unsigned int &offset) { Object::setData(d); checkSize(l); length=l; offset+=l; }
+   void setDataAndOffset(CharArray d, int l, int &offset) { Object::setData(d); checkSize(l); length=l; offset+=l; }
    virtual int getLength() { return length; }
 private:
    int length;
@@ -384,7 +387,7 @@ typedef uint64_t  SixtyFourBit;
 template <typename T> class TypeLoop : public Loop {
 public:
    int getCount() { return getLength()/sizeof(T); }
-   T operator[](const unsigned int index) const
+   T operator[](const int index) const
       {
          switch (sizeof(T)) {
          case 1:
@@ -411,6 +414,12 @@ public:
    MHP_DescriptorLoop() { domain=MHP; }
 };
 
+//Premiere Content Information Table
+class PCIT_DescriptorLoop : public DescriptorLoop {
+public:
+   PCIT_DescriptorLoop() { domain=PCIT; }
+};
+
 //The content of the ExtendedEventDescriptor may be split over several
 //descriptors if the text is longer than 256 bytes.
 //The following classes provide base functionality to handle this case.
