@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menuitems.c 1.24 2005/11/12 12:22:10 kls Exp $
+ * $Id: menuitems.c 1.44 2006/04/25 15:59:02 kls Exp $
  */
 
 #include "menuitems.h"
@@ -15,48 +15,58 @@
 #include "skins.h"
 #include "status.h"
 
-const char *FileNameChars = " abcdefghijklmnopqrstuvwxyz0123456789-.#~";
+#define AUTO_ADVANCE_TIMEOUT  1500 // ms before auto advance when entering characters via numeric keys
+
+const char *FileNameChars = " abcdefghijklmnopqrstuvwxyz0123456789-.#~,/_@";
 
 // --- cMenuEditItem ---------------------------------------------------------
 
 cMenuEditItem::cMenuEditItem(const char *Name)
 {
-  name = strdup(Name);
-  value = NULL;
+  name = strdup(Name ? Name : "???");
 }
 
 cMenuEditItem::~cMenuEditItem()
 {
   free(name);
-  free(value);
 }
 
 void cMenuEditItem::SetValue(const char *Value)
 {
-  free(value);
-  value = strdup(Value);
   char *buffer = NULL;
-  asprintf(&buffer, "%s:\t%s", name, value);
+  asprintf(&buffer, "%s:\t%s", name, Value);
   SetText(buffer, false);
   cStatus::MsgOsdCurrentItem(buffer);
 }
 
 // --- cMenuEditIntItem ------------------------------------------------------
 
-cMenuEditIntItem::cMenuEditIntItem(const char *Name, int *Value, int Min, int Max)
+cMenuEditIntItem::cMenuEditIntItem(const char *Name, int *Value, int Min, int Max, const char *MinString, const char *MaxString)
 :cMenuEditItem(Name)
 {
   value = Value;
   min = Min;
   max = Max;
+  minString = MinString;
+  maxString = MaxString;
+  if (*value < min)
+     *value = min;
+  else if (*value > max)
+     *value = max;
   Set();
 }
 
 void cMenuEditIntItem::Set(void)
 {
-  char buf[16];
-  snprintf(buf, sizeof(buf), "%d", *value);
-  SetValue(buf);
+  if (minString && *value == min)
+     SetValue(minString);
+  else if (maxString && *value == max)
+     SetValue(maxString);
+  else {
+     char buf[16];
+     snprintf(buf, sizeof(buf), "%d", *value);
+     SetValue(buf);
+     }
 }
 
 eOSState cMenuEditIntItem::ProcessKey(eKeys Key)
@@ -70,10 +80,10 @@ eOSState cMenuEditIntItem::ProcessKey(eKeys Key)
        case kNone: break;
        case k0 ... k9:
             if (fresh) {
-               *value = 0;
+               newValue = 0;
                fresh = false;
                }
-            newValue = *value * 10 + (Key - k0);
+            newValue = newValue * 10 + (Key - k0);
             break;
        case kLeft: // TODO might want to increase the delta if repeated quickly?
             newValue = *value - 1;
@@ -88,7 +98,7 @@ eOSState cMenuEditIntItem::ProcessKey(eKeys Key)
             if (*value > max) { *value = max; Set(); }
             return state;
        }
-     if ((!fresh || min <= newValue) && newValue <= max) {
+     if (newValue != *value && (!fresh || min <= newValue) && newValue <= max) {
         *value = newValue;
         Set();
         }
@@ -116,7 +126,7 @@ void cMenuEditBoolItem::Set(void)
 
 // --- cMenuEditBitItem ------------------------------------------------------
 
-cMenuEditBitItem::cMenuEditBitItem(const char *Name, int *Value, int Mask, const char *FalseString, const char *TrueString)
+cMenuEditBitItem::cMenuEditBitItem(const char *Name, uint *Value, uint Mask, const char *FalseString, const char *TrueString)
 :cMenuEditBoolItem(Name, &bit, FalseString, TrueString)
 {
   value = Value;
@@ -191,7 +201,7 @@ cMenuEditChrItem::cMenuEditChrItem(const char *Name, char *Value, const char *Al
 :cMenuEditItem(Name)
 {
   value = Value;
-  allowed = strdup(Allowed);
+  allowed = strdup(Allowed ? Allowed : "");
   current = strchr(allowed, *Value);
   if (!current)
      current = allowed;
@@ -237,37 +247,58 @@ eOSState cMenuEditChrItem::ProcessKey(eKeys Key)
 cMenuEditStrItem::cMenuEditStrItem(const char *Name, char *Value, int Length, const char *Allowed)
 :cMenuEditItem(Name)
 {
+  orgValue = NULL;
   value = Value;
   length = Length;
-  allowed = strdup(Allowed);
+  allowed = strdup(Allowed ? Allowed : "");
   pos = -1;
   insert = uppercase = false;
   newchar = true;
+  charMap = tr(" 0\t-.#~,/_@1\tabc2\tdef3\tghi4\tjkl5\tmno6\tpqrs7\ttuv8\twxyz9");
+  currentChar = NULL;
+  lastKey = kNone;
   Set();
 }
 
 cMenuEditStrItem::~cMenuEditStrItem()
 {
+  free(orgValue);
   free(allowed);
 }
 
 void cMenuEditStrItem::SetHelpKeys(void)
 {
-  if (pos >= 0)
-     cSkinDisplay::Current()->SetButtons(tr("ABC/abc"), tr(insert ? "Overwrite" : "Insert"), tr("Delete"));
+  if (InEditMode())
+     cSkinDisplay::Current()->SetButtons(tr("Button$ABC/abc"), tr(insert ? "Button$Overwrite" : "Button$Insert"), tr("Button$Delete"));
   else
      cSkinDisplay::Current()->SetButtons(NULL);
 }
 
+void cMenuEditStrItem::AdvancePos(void)
+{
+  if (pos < length - 2 && pos < int(strlen(value)) ) {
+     if (++pos >= int(strlen(value))) {
+        if (pos >= 2 && value[pos - 1] == ' ' && value[pos - 2] == ' ')
+           pos--; // allow only two blanks at the end
+        else {
+           value[pos] = ' ';
+           value[pos + 1] = 0;
+           }
+        }
+     }
+  newchar = true;
+  if (!insert && isalpha(value[pos]))
+     uppercase = isupper(value[pos]);
+}
+
 void cMenuEditStrItem::Set(void)
 {
   char buf[1000];
-  const char *fmt = insert && newchar ? "[]%c%s" : "[%c]%s";
 
-  if (pos >= 0) {
+  if (InEditMode()) {
      const cFont *font = cFont::GetFont(fontOsd);
      strncpy(buf, value, pos);
-     snprintf(buf + pos, sizeof(buf) - pos - 2, fmt, *(value + pos), value + pos + 1);
+     snprintf(buf + pos, sizeof(buf) - pos - 2, insert && newchar ? "[]%c%s" : "[%c]%s", *(value + pos), value + pos + 1);
      int width = cSkinDisplay::Current()->EditableWidth();
      if (font->Width(buf) <= width) {
         // the whole buffer fits on the screen
@@ -320,9 +351,19 @@ char cMenuEditStrItem::Inc(char c, bool Up)
 
 eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
 {
+  bool SameKey = NORMALKEY(Key) == lastKey;
+  if (Key != kNone)
+     lastKey = NORMALKEY(Key);
+  else if (!newchar && k0 <= lastKey && lastKey <= k9 && autoAdvanceTimeout.TimedOut()) {
+     AdvancePos();
+     newchar = true;
+     currentChar = NULL;
+     Set();
+     return osContinue;
+     }
   switch (Key) {
     case kRed:   // Switch between upper- and lowercase characters
-                 if (pos >= 0) {
+                 if (InEditMode()) {
                     if (!insert || !newchar) {
                        uppercase = !uppercase;
                        value[pos] = uppercase ? toupper(value[pos]) : tolower(value[pos]);
@@ -332,7 +373,7 @@ eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
                     return osUnknown;
                  break;
     case kGreen: // Toggle insert/overwrite modes
-                 if (pos >= 0) {
+                 if (InEditMode()) {
                     insert = !insert;
                     newchar = true;
                     SetHelpKeys();
@@ -342,7 +383,7 @@ eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
                  break;
     case kYellow|k_Repeat:
     case kYellow: // Remove the character at current position; in insert mode it is the character to the right of cursor
-                 if (pos >= 0) {
+                 if (InEditMode()) {
                     if (strlen(value) > 1) {
                        if (!insert || pos < int(strlen(value)) - 1)
                           memmove(value + pos, value + pos + 1, strlen(value) - pos);
@@ -361,7 +402,7 @@ eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
                  break;
     case kBlue|k_Repeat:
     case kBlue:  // consume the key only if in edit-mode
-                 if (pos >= 0)
+                 if (InEditMode())
                     ;
                  else
                     return osUnknown;
@@ -376,26 +417,16 @@ eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
                     uppercase = isupper(value[pos]);
                  break;
     case kRight|k_Repeat:
-    case kRight: if (pos < length - 2 && pos < int(strlen(value)) ) {
-                    if (++pos >= int(strlen(value))) {
-                       if (pos >= 2 && value[pos - 1] == ' ' && value[pos - 2] == ' ')
-                          pos--; // allow only two blanks at the end
-                       else {
-                          value[pos] = ' ';
-                          value[pos + 1] = 0;
-                          }
-                       }
-                    }
-                 newchar = true;
-                 if (!insert && isalpha(value[pos]))
-                    uppercase = isupper(value[pos]);
-                 if (pos == 0)
+    case kRight: AdvancePos();
+                 if (pos == 0) {
+                    orgValue = strdup(value);
                     SetHelpKeys();
+                    }
                  break;
     case kUp|k_Repeat:
     case kUp:
     case kDown|k_Repeat:
-    case kDown:  if (pos >= 0) {
+    case kDown:  if (InEditMode()) {
                     if (insert && newchar) {
                        // create a new character in insert mode
                        if (int(strlen(value)) < length - 1) {
@@ -412,7 +443,50 @@ eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
                  else
                     return cMenuEditItem::ProcessKey(Key);
                  break;
-    case kOk:    if (pos >= 0) {
+    case k0|k_Repeat ... k9|k_Repeat:
+    case k0 ... k9: {
+                 if (InEditMode()) {
+                    if (!SameKey) {
+                       if (!newchar)
+                          AdvancePos();
+                       currentChar = NULL;
+                       }
+                    if (insert && newchar) {
+                       // create a new character in insert mode
+                       if (int(strlen(value)) < length - 1) {
+                          memmove(value + pos + 1, value + pos, strlen(value) - pos + 1);
+                          value[pos] = ' ';
+                          }
+                       }
+                    if (!currentChar || !*currentChar || *currentChar == '\t') {
+                       // find the beginning of the character map entry for Key
+                       int n = NORMALKEY(Key) - k0;
+                       currentChar = charMap;
+                       while (n > 0 && *currentChar) {
+                             if (*currentChar++ == '\t')
+                                n--;
+                             }
+                       }
+                    if (*currentChar && *currentChar != '\t') {
+                       value[pos] = *currentChar;
+                       if (uppercase)
+                          value[pos] = toupper(value[pos]);
+                       currentChar++;
+                       }
+                    newchar = false;
+                    autoAdvanceTimeout.Set(AUTO_ADVANCE_TIMEOUT);
+                    }
+                 else
+                    return cMenuEditItem::ProcessKey(Key);
+                 }
+                 break;
+    case kBack:
+    case kOk:    if (InEditMode()) {
+                    if (Key == kBack && orgValue) {
+                       strcpy(value, orgValue);
+                       free(orgValue);
+                       orgValue = NULL;
+                       }
                     pos = -1;
                     newchar = true;
                     stripspace(value);
@@ -420,7 +494,7 @@ eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
                     break;
                     }
                  // run into default
-    default:     if (pos >= 0 && BASICKEY(Key) == kKbd) {
+    default:     if (InEditMode() && BASICKEY(Key) == kKbd) {
                     int c = KEYKBD(Key);
                     if (c <= 0xFF) {
                        const char *p = strchr(allowed, tolower(c));
@@ -479,18 +553,23 @@ void cMenuEditStraItem::Set(void)
 
 // --- cMenuEditChanItem -----------------------------------------------------
 
-cMenuEditChanItem::cMenuEditChanItem(const char *Name, int *Value)
-:cMenuEditIntItem(Name, Value, 1, Channels.MaxNumber())
+cMenuEditChanItem::cMenuEditChanItem(const char *Name, int *Value, const char *NoneString)
+:cMenuEditIntItem(Name, Value, NoneString ? 0 : 1, Channels.MaxNumber())
 {
+  noneString = NoneString;
   Set();
 }
 
 void cMenuEditChanItem::Set(void)
 {
-  char buf[255];
-  cChannel *channel = Channels.GetByNumber(*value);
-  snprintf(buf, sizeof(buf), "%d %s", *value, channel ? channel->Name() : "");
-  SetValue(buf);
+  if (*value > 0) {
+     char buf[255];
+     cChannel *channel = Channels.GetByNumber(*value);
+     snprintf(buf, sizeof(buf), "%d %s", *value, channel ? channel->Name() : "");
+     SetValue(buf);
+     }
+  else if (noneString)
+     SetValue(noneString);
 }
 
 eOSState cMenuEditChanItem::ProcessKey(eKeys Key)
@@ -504,10 +583,11 @@ eOSState cMenuEditChanItem::ProcessKey(eKeys Key)
     case kRight:
                  {
                    cChannel *channel = Channels.GetByNumber(*value + delta, delta);
-                   if (channel) {
+                   if (channel)
                       *value = channel->Number();
-                      Set();
-                      }
+                   else if (delta < 0 && noneString)
+                      *value = 0;
+                   Set();
                  }
                  break;
     default: return cMenuEditIntItem::ProcessKey(Key);
@@ -518,11 +598,11 @@ eOSState cMenuEditChanItem::ProcessKey(eKeys Key)
 // --- cMenuEditTranItem -----------------------------------------------------
 
 cMenuEditTranItem::cMenuEditTranItem(const char *Name, int *Value, int *Source)
-:cMenuEditChanItem(Name, Value)
+:cMenuEditChanItem(Name, &number, "-")
 {
   number = 0;
   source = Source;
-  transponder = *Value;
+  transponder = Value;
   cChannel *channel = Channels.First();
   while (channel) {
         if (!channel->GroupSep() && *source == channel->Source() && ISTRANSPONDER(channel->Transponder(), *Value)) {
@@ -531,22 +611,21 @@ cMenuEditTranItem::cMenuEditTranItem(const char *Name, int *Value, int *Source)
            }
         channel = (cChannel *)channel->Next();
         }
-  *Value = number;
   Set();
-  *Value = transponder;
 }
 
 eOSState cMenuEditTranItem::ProcessKey(eKeys Key)
 {
-  *value = number;
   eOSState state = cMenuEditChanItem::ProcessKey(Key);
-  number = *value;
-  cChannel *channel = Channels.GetByNumber(*value);
+  cChannel *channel = Channels.GetByNumber(number);
   if (channel) {
      *source = channel->Source();
-     transponder = channel->Transponder();
+     *transponder = channel->Transponder();
+     }
+  else {
+     *source = 0;
+     *transponder = 0;
      }
-  *value = transponder;
   return state;
 }
 
@@ -667,6 +746,7 @@ eOSState cMenuEditDateItem::ProcessKey(eKeys Key)
               }
            else {
               *weekdays = days[cTimer::GetWDay(*value)];
+              dayindex = FindDayIndex(*weekdays);
               oldvalue = *value;
               *value = 0;
               }
