@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.c 1.124 2005/11/04 14:19:44 kls Exp $
+ * $Id: recording.c 1.148 2006/04/29 13:22:20 kls Exp $
  */
 
 #include "recording.h"
@@ -48,8 +48,8 @@
 
 #define MINDISKSPACE 1024 // MB
 
-#define DELETEDLIFETIME     1 // hours after which a deleted recording will be actually removed
-#define REMOVECHECKDELTA 3600 // seconds between checks for removing deleted files
+#define REMOVECHECKDELTA   60 // seconds between checks for removing deleted files
+#define DELETEDLIFETIME   300 // seconds after which a deleted recording will be actually removed
 #define DISKCHECKDELTA    100 // seconds between checks for free disk space
 #define REMOVELATENCY      10 // seconds to wait until next check after removing a file
 
@@ -58,54 +58,80 @@
 
 #define MAX_SUBTITLE_LENGTH  40
 
+#define MAX_LINK_LEVEL  6
+
 bool VfatFileSystem = false;
 
-static cRecordings DeletedRecordings(true);
+cRecordings DeletedRecordings(true);
+
+// --- cRemoveDeletedRecordingsThread ----------------------------------------
+
+class cRemoveDeletedRecordingsThread : public cThread {
+protected:
+  virtual void Action(void);
+public:
+  cRemoveDeletedRecordingsThread(void);
+  };
+
+cRemoveDeletedRecordingsThread::cRemoveDeletedRecordingsThread(void)
+:cThread("remove deleted recordings")
+{
+}
+
+void cRemoveDeletedRecordingsThread::Action(void)
+{
+  // Make sure only one instance of VDR does this:
+  cLockFile LockFile(VideoDirectory);
+  if (LockFile.Lock()) {
+     bool deleted = false;
+     cThreadLock DeletedRecordingsLock(&DeletedRecordings);
+     for (cRecording *r = DeletedRecordings.First(); r; ) {
+         if (r->deleted && time(NULL) - r->deleted > DELETEDLIFETIME) {
+            cRecording *next = DeletedRecordings.Next(r);
+            r->Remove();
+            DeletedRecordings.Del(r);
+            r = next;
+            deleted = true;
+            continue;
+            }
+         r = DeletedRecordings.Next(r);
+         }
+     if (deleted)
+        RemoveEmptyVideoDirectories();
+     }
+}
+
+static cRemoveDeletedRecordingsThread RemoveDeletedRecordingsThread;
+
+// ---
 
 void RemoveDeletedRecordings(void)
 {
   static time_t LastRemoveCheck = 0;
-  if (LastRemoveCheck == 0) {
-     DeletedRecordings.Update();
-     LastRemoveCheck = time(NULL) - REMOVECHECKDELTA * 9 / 10;
-     }
-  else if (time(NULL) - LastRemoveCheck > REMOVECHECKDELTA) {
-     // Make sure only one instance of VDR does this:
-     cLockFile LockFile(VideoDirectory);
-     if (!LockFile.Lock())
-        return;
-     // Remove the oldest file that has been "deleted":
-     cThreadLock DeletedRecordingsLock(&DeletedRecordings);
-     if (DeletedRecordings.Count()) {
-        cRecording *r = DeletedRecordings.First();
-        cRecording *r0 = r;
-        while (r) {
-              if (r->start < r0->start)
-                 r0 = r;
-              r = DeletedRecordings.Next(r);
-              }
-        if (r0 && time(NULL) - r0->start > DELETEDLIFETIME * 3600) {
-           r0->Remove();
-           DeletedRecordings.Del(r0);
-           RemoveEmptyVideoDirectories();
-           LastRemoveCheck += REMOVELATENCY;
-           return;
-           }
+  if (time(NULL) - LastRemoveCheck > REMOVECHECKDELTA) {
+     if (!RemoveDeletedRecordingsThread.Active()) {
+        cThreadLock DeletedRecordingsLock(&DeletedRecordings);
+        for (cRecording *r = DeletedRecordings.First(); r; r = DeletedRecordings.Next(r)) {
+            if (r->deleted && time(NULL) - r->deleted > DELETEDLIFETIME) {
+               RemoveDeletedRecordingsThread.Start();
+               break;
+               }
+            }
         }
-     else
-        DeletedRecordings.Update();
      LastRemoveCheck = time(NULL);
      }
 }
 
-void AssertFreeDiskSpace(int Priority)
+void AssertFreeDiskSpace(int Priority, bool Force)
 {
+  static cMutex Mutex;
+  cMutexLock MutexLock(&Mutex);
   // With every call to this function we try to actually remove
   // a file, or mark a file for removal ("delete" it), so that
   // it will get removed during the next call.
   static time_t LastFreeDiskCheck = 0;
   int Factor = (Priority == -1) ? 10 : 1;
-  if (time(NULL) - LastFreeDiskCheck > DISKCHECKDELTA / Factor) {
+  if (Force || time(NULL) - LastFreeDiskCheck > DISKCHECKDELTA / Factor) {
      if (!VideoFileSpaceAvailable(MINDISKSPACE)) {
         // Make sure only one instance of VDR does this:
         cLockFile LockFile(VideoDirectory);
@@ -160,13 +186,13 @@ void AssertFreeDiskSpace(int Priority)
            }
         // Unable to free disk space, but there's nothing we can do about that...
         isyslog("...no old recording found, giving up");
-        Interface->Confirm(tr("Low disk space!"), 30);
+        Skins.QueueMessage(mtWarning, tr("Low disk space!"), 5, -1);
         }
      LastFreeDiskCheck = time(NULL);
      }
 }
 
-// --- cResumeFile ------------------------------------------------------------
+// --- cResumeFile -----------------------------------------------------------
 
 cResumeFile::cResumeFile(const char *FileName)
 {
@@ -233,20 +259,51 @@ void cResumeFile::Delete(void)
 
 // --- cRecordingInfo --------------------------------------------------------
 
-cRecordingInfo::cRecordingInfo(tChannelID ChannelID, const cEvent *Event)
+cRecordingInfo::cRecordingInfo(const cChannel *Channel, const cEvent *Event)
 {
-  channelID = ChannelID;
-  if (Event) {
-     event = Event;
-     ownEvent = NULL;
+  channelID = Channel ? Channel->GetChannelID() : tChannelID::InvalidID;
+  ownEvent = Event ? NULL : new cEvent(0);
+  event = ownEvent ? ownEvent : Event;
+  aux = NULL;
+  if (Channel) {
+     // Since the EPG data's component records can carry only a single
+     // language code, let's see whether the channel's PID data has
+     // more information:
+     cComponents *Components = (cComponents *)event->Components();
+     if (!Components)
+        Components = new cComponents;
+     for (int i = 0; i < MAXAPIDS; i++) {
+         const char *s = Channel->Alang(i);
+         if (*s) {
+            tComponent *Component = Components->GetComponent(i, 2, 3);
+            if (!Component)
+               Components->SetComponent(Components->NumComponents(), 2, 3, s, NULL);
+            else if (strlen(s) > strlen(Component->language))
+               strn0cpy(Component->language, s, sizeof(Component->language));
+            }
+         }
+     // There's no "multiple languages" for Dolby Digital tracks, but
+     // we do the same procedure here, too, in case there is no component
+     // information at all:
+     for (int i = 0; i < MAXDPIDS; i++) {
+         const char *s = Channel->Dlang(i);
+         if (*s) {
+            tComponent *Component = Components->GetComponent(i, 2, 5);
+            if (!Component)
+               Components->SetComponent(Components->NumComponents(), 2, 5, s, NULL);
+            else if (strlen(s) > strlen(Component->language))
+               strn0cpy(Component->language, s, sizeof(Component->language));
+            }
+         }
+     if (Components != event->Components())
+        ((cEvent *)event)->SetComponents(Components);
      }
-  else
-     event = ownEvent = new cEvent(0);
 }
 
 cRecordingInfo::~cRecordingInfo()
 {
   delete ownEvent;
+  free(aux);
 }
 
 void cRecordingInfo::SetData(const char *Title, const char *ShortText, const char *Description)
@@ -259,6 +316,12 @@ void cRecordingInfo::SetData(const char *Title, const char *ShortText, const cha
      ((cEvent *)event)->SetDescription(Description);
 }
 
+void cRecordingInfo::SetAux(const char *Aux)
+{
+  free(aux);
+  aux = Aux ? strdup(Aux) : NULL;
+}
+
 bool cRecordingInfo::Read(FILE *f)
 {
   if (ownEvent) {
@@ -277,6 +340,26 @@ bool cRecordingInfo::Read(FILE *f)
                             channelID = tChannelID::FromString(t);
                        }
                        break;
+             case 'E': {
+                         unsigned int EventID;
+                         time_t StartTime;
+                         int Duration;
+                         unsigned int TableID = 0;
+                         unsigned int Version = 0xFF;
+                         int n = sscanf(t, "%u %ld %d %X %X", &EventID, &StartTime, &Duration, &TableID, &Version);
+                         if (n >= 3 && n <= 5) {
+                            ownEvent->SetEventID(EventID);
+                            ownEvent->SetStartTime(StartTime);
+                            ownEvent->SetDuration(Duration);
+                            ownEvent->SetTableID(TableID);
+                            ownEvent->SetVersion(Version);
+                            }
+                       }
+                       break;
+             case '@': free(aux);
+                       aux = strdup(t);
+                       break;
+             case '#': break; // comments are ignored
              default: if (!ownEvent->Parse(s)) {
                          esyslog("ERROR: EPG data problem in line %d", line);
                          return false;
@@ -294,6 +377,8 @@ bool cRecordingInfo::Write(FILE *f, const char *Prefix) const
   if (channelID.Valid())
      fprintf(f, "%sC %s\n", Prefix, *channelID.ToString());
   event->Dump(f, Prefix, true);
+  if (aux)
+     fprintf(f, "%s@ %s\n", Prefix, aux);
   return true;
 }
 
@@ -304,7 +389,10 @@ bool cRecordingInfo::Write(FILE *f, const char *Prefix) const
 struct tCharExchange { char a; char b; };
 tCharExchange CharExchange[] = {
   { '~',  '/'    },
+  { '/',  '~'    },
   { ' ',  '_'    },
+  // backwards compatibility:
+  { '\'', '\''   },
   { '\'', '\x01' },
   { '/',  '\x02' },
   { 0, 0 }
@@ -400,6 +488,8 @@ cRecording::cRecording(cTimer *Timer, const cEvent *Event)
   sortBuffer = NULL;
   fileName = NULL;
   name = NULL;
+  fileSizeMB = -1; // unknown
+  deleted = 0;
   // set up the actual name:
   const char *Title = Event ? Event->Title() : NULL;
   const char *Subtitle = Event ? Event->ShortText() : NULL;
@@ -442,17 +532,15 @@ cRecording::cRecording(cTimer *Timer, const cEvent *Event)
   priority = Timer->Priority();
   lifetime = Timer->Lifetime();
   // handle info:
-  info = new cRecordingInfo(Timer->Channel()->GetChannelID(), Event);
-  // this is a somewhat ugly hack to get the 'summary' information from the
-  // timer into the recording info, but it saves us from having to actually
-  // copy the entire event data:
-  if (!isempty(Timer->Summary()))
-     info->SetData(isempty(info->Title()) ? Timer->File() : NULL, NULL, Timer->Summary());
+  info = new cRecordingInfo(Timer->Channel(), Event);
+  info->SetAux(Timer->Aux());
 }
 
 cRecording::cRecording(const char *FileName)
 {
   resume = RESUME_NOT_INITIALIZED;
+  fileSizeMB = -1; // unknown
+  deleted = 0;
   titleBuffer = NULL;
   sortBuffer = NULL;
   fileName = strdup(FileName);
@@ -525,7 +613,7 @@ cRecording::cRecording(const char *FileName)
               // so assume the short text is missing and concatenate
               // line 1 and line 2 to be the long text:
               int len = strlen(data[1]);
-              if (len > 80) { 
+              if (len > 80) {
                  data[1] = (char *)realloc(data[1], len + 1 + strlen(data[2]) + 1);
                  strcat(data[1], "\n");
                  strcat(data[1], data[2]);
@@ -579,6 +667,7 @@ char *cRecording::SortName(void) const
 {
   if (!sortBuffer) {
      char *s = StripEpisodeName(strdup(FileName() + strlen(VideoDirectory) + 1));
+     strreplace(s, '/', 'a'); // some locales ignore '/' when sorting
      int l = strxfrm(NULL, s, 0) + 1;
      sortBuffer = MALLOC(char, l);
      strxfrm(sortBuffer, s, l);
@@ -714,15 +803,20 @@ bool cRecording::Delete(void)
   bool result = true;
   char *NewName = strdup(FileName());
   char *ext = strrchr(NewName, '.');
-  if (strcmp(ext, RECEXT) == 0) {
+  if (ext && strcmp(ext, RECEXT) == 0) {
      strncpy(ext, DELEXT, strlen(ext));
      if (access(NewName, F_OK) == 0) {
         // the new name already exists, so let's remove that one first:
-        isyslog("removing recording %s", NewName);
+        isyslog("removing recording '%s'", NewName);
         RemoveVideoFile(NewName);
         }
-     isyslog("deleting recording %s", FileName());
-     result = RenameVideoFile(FileName(), NewName);
+     isyslog("deleting recording '%s'", FileName());
+     if (access(FileName(), F_OK) == 0)
+        result = RenameVideoFile(FileName(), NewName);
+     else {
+        isyslog("recording '%s' vanished", FileName());
+        result = true; // well, we were going to delete it, anyway
+        }
      }
   free(NewName);
   return result;
@@ -785,7 +879,7 @@ void cRecordings::Refresh(bool Foreground)
   ScanVideoDir(VideoDirectory, Foreground);
 }
 
-void cRecordings::ScanVideoDir(const char *DirName, bool Foreground)
+void cRecordings::ScanVideoDir(const char *DirName, bool Foreground, int LinkLevel)
 {
   cReadDir d(DirName);
   struct dirent *e;
@@ -795,7 +889,13 @@ void cRecordings::ScanVideoDir(const char *DirName, bool Foreground)
            asprintf(&buffer, "%s/%s", DirName, e->d_name);
            struct stat st;
            if (stat(buffer, &st) == 0) {
+              int Link = 0;
               if (S_ISLNK(st.st_mode)) {
+                 if (LinkLevel > MAX_LINK_LEVEL) {
+                    isyslog("max link level exceeded - not scanning %s", buffer);
+                    continue;
+                    }
+                 Link = 1;
                  char *old = buffer;
                  buffer = ReadLink(old);
                  free(old);
@@ -814,12 +914,16 @@ void cRecordings::ScanVideoDir(const char *DirName, bool Foreground)
                        Add(r);
                        ChangeState();
                        Unlock();
+                       if (deleted) {
+                          r->fileSizeMB = DirSizeMB(buffer);
+                          r->deleted = time(NULL);
+                          }
                        }
                     else
                        delete r;
                     }
                  else
-                    ScanVideoDir(buffer, Foreground);
+                    ScanVideoDir(buffer, Foreground, LinkLevel + Link);
                  }
               }
            free(buffer);
@@ -883,12 +987,33 @@ void cRecordings::DelByName(const char *FileName)
   LOCK_THREAD;
   cRecording *recording = GetByName(FileName);
   if (recording) {
-     Del(recording);
+     cThreadLock DeletedRecordingsLock(&DeletedRecordings);
+     Del(recording, false);
+     char *ext = strrchr(recording->FileName(), '.');
+     if (ext) {
+        strncpy(ext, DELEXT, strlen(ext));
+        recording->fileSizeMB = DirSizeMB(recording->FileName());
+        recording->deleted = time(NULL);
+        DeletedRecordings.Add(recording);
+        }
+     else
+        delete recording;
      ChangeState();
      TouchUpdate();
      }
 }
 
+int cRecordings::TotalFileSizeMB(void)
+{
+  int size = 0;
+  LOCK_THREAD;
+  for (cRecording *recording = First(); recording; recording = Next(recording)) {
+      if (recording->fileSizeMB > 0)
+         size += recording->fileSizeMB;
+      }
+  return size;
+}
+
 void cRecordings::ResetResume(const char *ResumeFileName)
 {
   LOCK_THREAD;
@@ -935,7 +1060,7 @@ bool cMark::Parse(const char *s)
 
 bool cMark::Save(FILE *f)
 {
-  return fprintf(f, ToText()) > 0;
+  return fprintf(f, "%s", *ToText()) > 0;
 }
 
 // --- cMarks ----------------------------------------------------------------
@@ -1072,7 +1197,7 @@ cIndexFile::cIndexFile(const char *FileName, bool Record)
                        LOG_ERROR_STR(fileName);
                     }
                  else
-                    esyslog("ERROR: can't allocate %d bytes for index '%s'", size * sizeof(tIndex), fileName);
+                    esyslog("ERROR: can't allocate %zd bytes for index '%s'", size * sizeof(tIndex), fileName);
                  }
               }
            else
@@ -1246,6 +1371,11 @@ int cIndexFile::Get(uchar FileNumber, int FileOffset)
   return -1;
 }
 
+bool cIndexFile::IsStillRecording()
+{
+  return f >= 0;
+}
+
 // --- cFileName -------------------------------------------------------------
 
 #include <errno.h>
@@ -1306,7 +1436,7 @@ cUnbufferedFile *cFileName::Open(void)
 void cFileName::Close(void)
 {
   if (file) {
-     if ((record && CloseVideoFile(file) < 0) || (!record && file->Close() < 0))
+     if (CloseVideoFile(file) < 0)
         LOG_ERROR_STR(fileName);
      file = NULL;
      }
