@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: remote.c 1.45 2005/09/03 12:29:48 kls Exp $
+ * $Id: remote.c 1.50 2006/04/17 08:58:28 kls Exp $
  */
 
 #include "remote.h"
@@ -18,11 +18,13 @@
 
 // --- cRemote ---------------------------------------------------------------
 
-#define INITTIMEOUT 10000 // ms
+#define INITTIMEOUT   10000 // ms
+#define REPEATTIMEOUT  1000 // ms
 
 eKeys cRemote::keys[MaxKeys];
 int cRemote::in = 0;
 int cRemote::out = 0;
+cTimeMs cRemote::repeatTimeout;
 cRemote *cRemote::learning = NULL;
 char *cRemote::unknownCode = NULL;
 cMutex cRemote::mutex;
@@ -143,10 +145,23 @@ bool cRemote::Put(const char *Code, bool Repeat, bool Release)
   return false;
 }
 
-void cRemote::CallPlugin(const char *Plugin)
+bool cRemote::CallPlugin(const char *Plugin)
 {
-  plugin = Plugin;
-  Put(k_Plugin);
+  cMutexLock MutexLock(&mutex);
+  if (!plugin) {
+     plugin = Plugin;
+     Put(k_Plugin);
+     return true;
+     }
+  return false;
+}
+
+const char *cRemote::GetPlugin(void)
+{
+  cMutexLock MutexLock(&mutex);
+  const char *p = plugin;
+  plugin = NULL;
+  return p;
 }
 
 bool cRemote::HasKeys(void)
@@ -163,9 +178,11 @@ eKeys cRemote::Get(int WaitMs, char **UnknownCode)
          eKeys k = keys[out];
          if (++out >= MaxKeys)
             out = 0;
+         if ((k & k_Repeat) != 0)
+            repeatTimeout.Set(REPEATTIMEOUT);
          return k;
          }
-      else if (!WaitMs || !keyPressed.TimedWait(mutex, WaitMs)) {
+      else if (!WaitMs || !keyPressed.TimedWait(mutex, WaitMs) && repeatTimeout.TimedOut()) {
          if (learning && UnknownCode) {
             *UnknownCode = unknownCode;
             unknownCode = NULL;
@@ -262,43 +279,73 @@ int cKbdRemote::MapCodeToFunc(uint64 Code)
   return (Code <= 0xFF) ? Code : kfNone;
 }
 
-void cKbdRemote::Action(void)
+int cKbdRemote::ReadKey(void)
 {
   cPoller Poller(STDIN_FILENO);
+  if (Poller.Poll(50)) {
+     uchar ch = 0;
+     int r = safe_read(STDIN_FILENO, &ch, 1);
+     if (r == 1)
+        return ch;
+     if (r < 0)
+        LOG_ERROR_STR("cKbdRemote");
+     }
+  return -1;
+}
+
+uint64 cKbdRemote::ReadKeySequence(void)
+{
+  uint64 k = 0;
+  int key1;
+
+  if ((key1 = ReadKey()) >= 0) {
+     k = key1;
+     if (key1 == 0x1B) {
+         // Start of escape sequence
+         if ((key1 = ReadKey()) >= 0) {
+            k <<= 8;
+            k |= key1 & 0xFF;
+            switch (key1) {
+              case 0x4F: // 3-byte sequence
+                   if ((key1 = ReadKey()) >= 0) {
+                      k <<= 8;
+                      k |= key1 & 0xFF;
+                      }
+                   break;
+              case 0x5B: // 3- or more-byte sequence
+                   if ((key1 = ReadKey()) >= 0) {
+                      k <<= 8;
+                      k |= key1 & 0xFF;
+                      switch (key1) {
+                        case 0x31 ... 0x3F: // more-byte sequence
+                        case 0x5B: // strange, may apparently occur
+                             do {
+                                if ((key1 = ReadKey()) < 0)
+                                   break; // Sequence ends here
+                                k <<= 8;
+                                k |= key1 & 0xFF;
+                                } while (key1 != 0x7E);
+                             break;
+                        }
+                      }
+                   break;
+              }
+            }
+        }
+     }
+  return k;
+}
+
+void cKbdRemote::Action(void)
+{
   while (Running()) {
-        if (Poller.Poll(100)) {
-           uint64 Command = 0;
-           uint i = 0;
-           while (Running() && i < sizeof(Command)) {
-                 uchar ch;
-                 int r = read(STDIN_FILENO, &ch, 1);
-                 if (r == 1) {
-                    Command <<= 8;
-                    Command |= ch;
-                    i++;
-                    }
-                 else if (r == 0) {
-                    // don't know why, but sometimes special keys that start with
-                    // 0x1B ('ESC') cause a short gap between the 0x1B and the rest
-                    // of their codes, so we'll need to wait some 100ms to see if
-                    // there is more coming up - or whether this really is the 'ESC'
-                    // key (if somebody knows how to clean this up, please let me know):
-                    if (Command == 0x1B && Poller.Poll(100))
-                       continue;
-                    if (Command) {
-                       if (rawMode || !Put(Command)) {
-                          int func = MapCodeToFunc(Command);
-                          if (func)
-                             Put(KBDKEY(func));
-                          }
-                       }
-                    break;
-                    }
-                 else {
-                    LOG_ERROR;
-                    break;
-                    }
-                 }
+        uint64 Command = ReadKeySequence();
+        if (Command) {
+           if (rawMode || !Put(Command)) {
+              int func = MapCodeToFunc(Command);
+              if (func)
+                 Put(KBDKEY(func));
+              }
            }
         }
 }
