@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: skins.c 1.6 2005/11/27 15:52:25 kls Exp $
+ * $Id: skins.c 1.9 2006/04/09 11:25:30 kls Exp $
  */
 
 #include "skins.h"
@@ -108,6 +108,16 @@ const char *cSkinDisplayMenu::GetTabbedText(const char *s, int Tab)
   return buffer;
 }
 
+int cSkinDisplayMenu::GetTextAreaWidth(void) const
+{
+  return 0;
+}
+
+const cFont *cSkinDisplayMenu::GetTextAreaFont(bool) const
+{
+  return NULL;
+}
+
 // --- cSkinDisplayReplay::cProgressBar --------------------------------------
 
 cSkinDisplayReplay::cProgressBar::cProgressBar(int Width, int Height, int Current, int Total, const cMarks *Marks, tColor ColorSeen, tColor ColorRest, tColor ColorSelected, tColor ColorMark, tColor ColorCurrent)
@@ -245,7 +255,16 @@ int cSkins::QueueMessage(eMessageType Type, const char *s, int Seconds, int Time
      return kNone;
      }
   if (isempty(s)) {
-     dsyslog("cSkins::QueueMessage() called with empty message - ignored!");
+     if (!cThread::IsMainThread()) {
+        queueMessageMutex.Lock();
+        for (cSkinQueuedMessage *m = SkinQueuedMessages.Last(); m; m = SkinQueuedMessages.Prev(m)) {
+            if (m->threadId == cThread::ThreadId() && m->state == 0)
+               m->state = 2; // done
+            }
+        queueMessageMutex.Unlock();
+        }
+     else
+        dsyslog("cSkins::QueueMessage() called with empty message from main thread - ignored!");
      return kNone;
      }
   int k = kNone;
@@ -323,7 +342,7 @@ void cSkins::ProcessQueuedMessages(void)
          }
       else
          break;
-      } 
+      }
   queueMessageMutex.Unlock();
 }
 
