@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: skins.h 1.9 2005/11/27 15:41:44 kls Exp $
+ * $Id: skins.h 1.13 2006/04/09 11:23:35 kls Exp $
  */
 
 #ifndef __SKINS_H
@@ -160,6 +160,19 @@ public:
        ///< the Scroll() function will be called to drive scrolling that text if
        ///< necessary.
   //XXX ??? virtual void SetHelp(const char *Help) = 0;
+  virtual int GetTextAreaWidth(void) const;
+       ///< Returns the width in pixel of the area which is used to display text
+       ///< with SetText(). The width of the area is the width of the central area
+       ///< minus the width of any possibly displayed scroll-bar or other decoration.
+       ///< The default implementation returns 0. Therefore a caller of this method
+       ///< must be prepared to receive 0 if the plugin doesn't implement this method.
+  virtual const cFont *GetTextAreaFont(bool FixedFont) const;
+       ///< Returns a pointer to the font which is used to display text with SetText().
+       ///< The parameter FixedFont has the same meaning as in SetText(). The default
+       ///< implementation returns NULL. Therefore a caller of this method must be
+       ///< prepared to receive NULL if the plugin doesn't implement this method.
+       ///< The returned pointer is valid a long as the instance of cSkinDisplayMenu
+       ///< exists.
   };
 
 class cSkinDisplayReplay : public cSkinDisplay {
@@ -197,7 +210,7 @@ public:
        ///< take care that it is erased from the display when a Current string
        ///< _with_ ".ff" is followed by one without it.
   virtual void SetTotal(const char *Total) = 0;
-       ///< Sets the total length of the recording, as a user readable 
+       ///< Sets the total length of the recording, as a user readable
        ///< string if the form "h:mm:ss".
   virtual void SetJump(const char *Jump) = 0;
        ///< Sets the prompt that allows the user to enter a jump point.
@@ -316,7 +329,7 @@ public:
        ///< temporary cSkinDisplayMessage object.
        ///< The return value is the key pressed by the user. If no user input
        ///< has been received within Seconds (the default value of 0 results
-       ///< in the ///< value defined for "Message time" in the setup), kNone
+       ///< in the value defined for "Message time" in the setup), kNone
        ///< will be returned.
   int QueueMessage(eMessageType Type, const char *s, int Seconds = 0, int Timeout = 0);
        ///< Like Message(), but this function may be called from a background
@@ -340,7 +353,9 @@ public:
        ///< progress displays, where only the most recent message is actually
        ///< important.
        ///< Type may only be mtInfo, mtWarning or mtError. A call with mtStatus
-       ///< will be ignored, as will be one with an empty message.
+       ///< will be ignored. A call with an empty message from a background thread
+       ///< removes all queued messages from the calling thread. A call with
+       ///< an empty message from the main thread will be ignored.
   void ProcessQueuedMessages(void);
        ///< Processes the first queued message, if any.
   void Flush(void);
