@@ -10,7 +10,7 @@
  * and interact with the Video Disk Recorder - or write a full featured
  * graphical interface that sits on top of an SVDRP connection.
  *
- * $Id: svdrp.c 1.84 2005/11/27 15:29:28 kls Exp $
+ * $Id: svdrp.c 1.95 2006/04/17 09:02:23 kls Exp $
  */
 
 #include "svdrp.h"
@@ -201,10 +201,16 @@ const char *HelpPages[] = {
   "    Edit the recording with the given number. Before a recording can be\n"
   "    edited, an LSTR command must have been executed in order to retrieve\n"
   "    the recording numbers.",
-  "GRAB <filename> [ jpeg | pnm [ <quality> [ <sizex> <sizey> ] ] ]\n"
+  "GRAB <filename> [ <quality> [ <sizex> <sizey> ] ]\n"
   "    Grab the current frame and save it to the given file. Images can\n"
-  "    be stored as JPEG (default) or PNM, at the given quality (default\n"
-  "    is 'maximum', only applies to JPEG) and size (default is full screen).",
+  "    be stored as JPEG or PNM, depending on the given file name extension.\n"
+  "    The quality of the grabbed image can be in the range 0..100, where 100\n"
+  "    (the default) means \"best\" (only applies to JPEG). The size parameters\n"
+  "    define the size of the resulting image (default is full screen).\n"
+  "    If the file name is just an extension (.jpg, .jpeg or .pnm) the image\n"
+  "    data will be sent to the SVDRP connection encoded in base64. The same\n"
+  "    happens if '-' (a minus sign) is given as file name, in which case the\n"
+  "    image format defaults to JPEG.",
   "HELP [ <topic> ]\n"
   "    The HELP command gives help info.",
   "HITK [ <key> ]\n"
@@ -223,9 +229,11 @@ const char *HelpPages[] = {
   "LSTR [ <number> ]\n"
   "    List recordings. Without option, all recordings are listed. Otherwise\n"
   "    the information for the given recording is listed.",
-  "LSTT [ <number> ]\n"
+  "LSTT [ <number> ] [ id ]\n"
   "    List timers. Without option, all timers are listed. Otherwise\n"
-  "    only the given timer is listed.",
+  "    only the given timer is listed. If the keyword 'id' is given, the\n"
+  "    channels will be listed with their unique channel ids instead of\n"
+  "    their numbers.",
   "MESG <message>\n"
   "    Displays the given message on the OSD. The message will be queued\n"
   "    and displayed whenever this is suitable.\n",
@@ -307,6 +315,7 @@ const char *HelpPages[] = {
 
  214 Help message
  215 EPG or recording data record
+ 216 Image grab data (base 64)
  220 VDR service ready
  221 VDR service closing transmission channel
  250 Requested VDR action okay, completed
@@ -354,6 +363,8 @@ const char *GetHelpPage(const char *Cmd, const char **p)
   return NULL;
 }
 
+char *cSVDRP::grabImageDir = NULL;
+
 cSVDRP::cSVDRP(int Port)
 :socket(Port)
 {
@@ -450,7 +461,7 @@ void cSVDRP::PrintHelpTopics(const char **hp)
              q += sprintf(q, "%*s", -MAXHELPTOPIC, topic);
           }
       x = 0;
-      Reply(-214, buffer);
+      Reply(-214, "%s", buffer);
       }
 }
 
@@ -656,36 +667,50 @@ void cSVDRP::CmdGRAB(const char *Option)
      const char *delim = " \t";
      char *strtok_next;
      FileName = strtok_r(p, delim, &strtok_next);
-     if ((p = strtok_r(NULL, delim, &strtok_next)) != NULL) {
-        if (strcasecmp(p, "JPEG") == 0)
+     // image type:
+     char *Extension = strrchr(FileName, '.');
+     if (Extension) {
+        if (strcasecmp(Extension, ".jpg") == 0 || strcasecmp(Extension, ".jpeg") == 0)
            Jpeg = true;
-        else if (strcasecmp(p, "PNM") == 0)
+        else if (strcasecmp(Extension, ".pnm") == 0)
            Jpeg = false;
         else {
-           Reply(501, "Unknown image type \"%s\"", p);
+           Reply(501, "Unknown image type \"%s\"", Extension + 1);
            return;
            }
+        if (Extension == FileName)
+           FileName = NULL;
         }
+     else if (strcmp(FileName, "-") == 0)
+        FileName = NULL;
+     // image quality (and obsolete type):
      if ((p = strtok_r(NULL, delim, &strtok_next)) != NULL) {
-        if (isnumber(p))
-           Quality = atoi(p);
-        else {
-           Reply(501, "Illegal quality \"%s\"", p);
-           return;
+        if (strcasecmp(p, "JPEG") == 0 || strcasecmp(p, "PNM") == 0) {
+           // tolerate for backward compatibility
+           p = strtok_r(NULL, delim, &strtok_next);
+           }
+        if (p) {
+           if (isnumber(p))
+              Quality = atoi(p);
+           else {
+              Reply(501, "Invalid quality \"%s\"", p);
+              return;
+              }
            }
         }
+     // image size:
      if ((p = strtok_r(NULL, delim, &strtok_next)) != NULL) {
         if (isnumber(p))
            SizeX = atoi(p);
         else {
-           Reply(501, "Illegal sizex \"%s\"", p);
+           Reply(501, "Invalid sizex \"%s\"", p);
            return;
            }
         if ((p = strtok_r(NULL, delim, &strtok_next)) != NULL) {
            if (isnumber(p))
               SizeY = atoi(p);
            else {
-              Reply(501, "Illegal sizey \"%s\"", p);
+              Reply(501, "Invalid sizey \"%s\"", p);
               return;
               }
            }
@@ -698,8 +723,70 @@ void cSVDRP::CmdGRAB(const char *Option)
         Reply(501, "Unexpected parameter \"%s\"", p);
         return;
         }
-     if (cDevice::PrimaryDevice()->GrabImage(FileName, Jpeg, Quality, SizeX, SizeY))
-        Reply(250, "Grabbed image %s", Option);
+     // canonicalize the file name:
+     char RealFileName[PATH_MAX];
+     if (FileName) {
+        if (grabImageDir) {
+           char *s = 0;
+           char *slash = strrchr(FileName, '/');
+           if (!slash) {
+              asprintf(&s, "%s/%s", grabImageDir, FileName);
+              FileName = s;
+              }
+           slash = strrchr(FileName, '/'); // there definitely is one
+           *slash = 0;
+           char *r = realpath(FileName, RealFileName);
+           *slash = '/';
+           if (!r) {
+              LOG_ERROR_STR(FileName);
+              Reply(501, "Invalid file name \"%s\"", FileName);
+              free(s);
+              return;
+              }
+           strcat(RealFileName, slash);
+           FileName = RealFileName;
+           free(s);
+           if (strncmp(FileName, grabImageDir, strlen(grabImageDir)) != 0) {
+              Reply(501, "Invalid file name \"%s\"", FileName);
+              return;
+              }
+           }
+        else {
+           Reply(550, "Grabbing to file not allowed (use \"GRAB -\" instead)");
+           return;
+           }
+        }
+     // actual grabbing:
+     int ImageSize;
+     uchar *Image = cDevice::PrimaryDevice()->GrabImage(ImageSize, Jpeg, Quality, SizeX, SizeY);
+     if (Image) {
+        if (FileName) {
+           int fd = open(FileName, O_WRONLY | O_CREAT | O_NOFOLLOW | O_TRUNC, DEFFILEMODE);
+           if (fd >= 0) {
+              if (safe_write(fd, Image, ImageSize) == ImageSize) {
+                 isyslog("grabbed image to %s", FileName);
+                 Reply(250, "Grabbed image %s", Option);
+                 }
+              else {
+                 LOG_ERROR_STR(FileName);
+                 Reply(451, "Can't write to '%s'", FileName);
+                 }
+              close(fd);
+              }
+           else {
+              LOG_ERROR_STR(FileName);
+              Reply(451, "Can't open '%s'", FileName);
+              }
+           }
+        else {
+           cBase64Encoder Base64(Image, ImageSize);
+           const char *s;
+           while ((s = Base64.NextLine()) != NULL)
+                 Reply(-216, "%s", s);
+           Reply(216, "Grabbed image %s", Option);
+           }
+        free(Image);
+        }
      else
         Reply(451, "Grab image failed");
      }
@@ -712,7 +799,7 @@ void cSVDRP::CmdHELP(const char *Option)
   if (*Option) {
      const char *hp = GetHelpPage(Option, HelpPages);
      if (hp)
-        Reply(214, hp);
+        Reply(214, "%s", hp);
      else {
         Reply(504, "HELP topic \"%s\" unknown", Option);
         return;
@@ -845,7 +932,7 @@ void cSVDRP::CmdLSTE(const char *Option)
                  else
                     Channel = Channels.GetByChannelID(tChannelID::FromString(Option));
                  if (Channel) {
-                    Schedule = Schedules->GetSchedule(Channel->GetChannelID());
+                    Schedule = Schedules->GetSchedule(Channel);
                     if (!Schedule) {
                        Reply(550, "No schedule found");
                        return;
@@ -923,22 +1010,38 @@ void cSVDRP::CmdLSTR(const char *Option)
 
 void cSVDRP::CmdLSTT(const char *Option)
 {
+  int Number = 0;
+  bool Id = false;
   if (*Option) {
-     if (isnumber(Option)) {
-        cTimer *timer = Timers.Get(strtol(Option, NULL, 10) - 1);
-        if (timer)
-           Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
-        else
-           Reply(501, "Timer \"%s\" not defined", Option);
-        }
+     char buf[strlen(Option) + 1];
+     strcpy(buf, Option);
+     const char *delim = " \t";
+     char *strtok_next;
+     char *p = strtok_r(buf, delim, &strtok_next);
+     while (p) {
+           if (isnumber(p))
+              Number = strtol(p, NULL, 10);
+           else if (strcasecmp(p, "ID") == 0)
+              Id = true;
+           else {
+              Reply(501, "Unknown option: \"%s\"", p);
+              return;
+              }
+           p = strtok_r(NULL, delim, &strtok_next);
+           }
+     }
+  if (Number) {
+     cTimer *timer = Timers.Get(Number - 1);
+     if (timer)
+        Reply(250, "%d %s", timer->Index() + 1, *timer->ToText(Id));
      else
-        Reply(501, "Error in timer number \"%s\"", Option);
+        Reply(501, "Timer \"%s\" not defined", Option);
      }
   else if (Timers.Count()) {
      for (int i = 0; i < Timers.Count(); i++) {
          cTimer *timer = Timers.Get(i);
         if (timer)
-           Reply(i < Timers.Count() - 1 ? -250 : 250, "%d %s", timer->Index() + 1, *timer->ToText());
+           Reply(i < Timers.Count() - 1 ? -250 : 250, "%d %s", timer->Index() + 1, *timer->ToText(Id));
         else
            Reply(501, "Timer \"%d\" not found", i + 1);
          }
@@ -1229,7 +1332,7 @@ void cSVDRP::CmdPLUG(const char *Option)
            if (*cmd && *option) {
               const char *hp = GetHelpPage(option, plugin->SVDRPHelpPages());
               if (hp) {
-                 Reply(-214, hp);
+                 Reply(-214, "%s", hp);
                  Reply(214, "End of HELP info");
                  }
               else
@@ -1248,14 +1351,16 @@ void cSVDRP::CmdPLUG(const char *Option)
               }
            }
         else if (strcasecmp(cmd, "MAIN") == 0) {
-           cRemote::CallPlugin(plugin->Name());
-           Reply(250, "Initiated call to main menu function of plugin \"%s\"", plugin->Name());
+           if (cRemote::CallPlugin(plugin->Name()))
+              Reply(250, "Initiated call to main menu function of plugin \"%s\"", plugin->Name());
+           else
+              Reply(550, "A plugin call is already pending - please try again later");
            }
         else {
            int ReplyCode = 900;
            cString s = plugin->SVDRPCommand(cmd, option, ReplyCode);
            if (s)
-              Reply(abs(ReplyCode), *s);
+              Reply(abs(ReplyCode), "%s", *s);
            else
               Reply(500, "Command unrecognized: \"%s\"", cmd);
            }
@@ -1277,7 +1382,7 @@ void cSVDRP::CmdPUTE(const char *Option)
 {
   delete PUTEhandler;
   PUTEhandler = new cPUTEhandler;
-  Reply(PUTEhandler->Status(), PUTEhandler->Message());
+  Reply(PUTEhandler->Status(), "%s", PUTEhandler->Message());
   if (PUTEhandler->Status() != 354)
      DELETENULL(PUTEhandler);
 }
@@ -1364,7 +1469,7 @@ void cSVDRP::Execute(char *Cmd)
   // handle PUTE data:
   if (PUTEhandler) {
      if (!PUTEhandler->Process(Cmd)) {
-        Reply(PUTEhandler->Status(), PUTEhandler->Message());
+        Reply(PUTEhandler->Status(), "%s", PUTEhandler->Message());
         DELETENULL(PUTEhandler);
         }
      return;
@@ -1482,4 +1587,10 @@ bool cSVDRP::Process(void)
   return false;
 }
 
+void cSVDRP::SetGrabImageDir(const char *GrabImageDir)
+{
+  free(grabImageDir);
+  grabImageDir = GrabImageDir ? strdup(GrabImageDir) : NULL;
+}
+
 //TODO more than one connection???
