@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.c 1.36 2005/09/09 15:22:33 kls Exp $
+ * $Id: timers.c 1.58 2006/04/21 15:12:49 kls Exp $
  */
 
 #include "timers.h"
@@ -12,6 +12,7 @@
 #include "channels.h"
 #include "device.h"
 #include "i18n.h"
+#include "libsi/si.h"
 #include "remote.h"
 
 // IMPORTANT NOTE: in the 'sscanf()' calls there is a blank after the '%d'
@@ -20,14 +21,15 @@
 
 // -- cTimer -----------------------------------------------------------------
 
-cTimer::cTimer(bool Instant, bool Pause)
+cTimer::cTimer(bool Instant, bool Pause, cChannel *Channel)
 {
   startTime = stopTime = 0;
+  lastSetEvent = 0;
   recording = pending = inVpsMargin = false;
   flags = tfNone;
   if (Instant)
      SetFlags(tfActive | tfInstant);
-  channel = Channels.GetByNumber(cDevice::CurrentChannel());
+  channel = Channel ? Channel : Channels.GetByNumber(cDevice::CurrentChannel());
   time_t t = time(NULL);
   struct tm tm_r;
   struct tm *now = localtime_r(&t, &tm_r);
@@ -41,7 +43,7 @@ cTimer::cTimer(bool Instant, bool Pause)
   priority = Pause ? Setup.PausePriority : Setup.DefaultPriority;
   lifetime = Pause ? Setup.PauseLifetime : Setup.DefaultLifetime;
   *file = 0;
-  summary = NULL;
+  aux = NULL;
   event = NULL;
   if (Instant && channel)
      snprintf(file, sizeof(file), "%s%s", Setup.MarkInstantRecord ? "@" : "", *Setup.NameInstantRecord ? Setup.NameInstantRecord : channel->Name());
@@ -50,6 +52,7 @@ cTimer::cTimer(bool Instant, bool Pause)
 cTimer::cTimer(const cEvent *Event)
 {
   startTime = stopTime = 0;
+  lastSetEvent = 0;
   recording = pending = inVpsMargin = false;
   flags = tfActive;
   if (Event->Vps() && Setup.UseVps)
@@ -76,21 +79,22 @@ cTimer::cTimer(const cEvent *Event)
   const char *Title = Event->Title();
   if (!isempty(Title))
      strn0cpy(file, Event->Title(), sizeof(file));
-  summary = NULL;
-  event = Event;
+  aux = NULL;
+  event = NULL; // let SetEvent() be called to get a log message
 }
 
 cTimer::~cTimer()
 {
-  free(summary);
+  free(aux);
 }
 
 cTimer& cTimer::operator= (const cTimer &Timer)
 {
   memcpy(this, &Timer, sizeof(*this));
-  if (summary)
-     summary = strdup(summary);
+  if (aux)
+     aux = strdup(aux);
   event = NULL;
+  lastSetEvent = 0;
   return *this;
 }
 
@@ -109,9 +113,7 @@ cString cTimer::ToText(bool UseChannelID)
 {
   char *buffer;
   strreplace(file, ':', '|');
-  strreplace(summary, '\n', '|');
-  asprintf(&buffer, "%d:%s:%s:%04d:%04d:%d:%d:%s:%s\n", flags, UseChannelID ? *Channel()->GetChannelID().ToString() : *itoa(Channel()->Number()), *PrintDay(day, weekdays), start, stop, priority, lifetime, file, summary ? summary : "");
-  strreplace(summary, '|', '\n');
+  asprintf(&buffer, "%u:%s:%s:%04d:%04d:%d:%d:%s:%s\n", flags, UseChannelID ? *Channel()->GetChannelID().ToString() : *itoa(Channel()->Number()), *PrintDay(day, weekdays), start, stop, priority, lifetime, file, aux ? aux : "");
   strreplace(file, '|', ':');
   return cString(buffer, true);
 }
@@ -119,7 +121,7 @@ cString cTimer::ToText(bool UseChannelID)
 cString cTimer::ToDescr(void) const
 {
   char *buffer;
-  asprintf(&buffer, "%d (%d %04d-%04d '%s')", Index() + 1, Channel()->Number(), start, stop, file);
+  asprintf(&buffer, "%d (%d %04d-%04d %s'%s')", Index() + 1, Channel()->Number(), start, stop, HasFlags(tfVps) ? "VPS " : "", file);
   return cString(buffer, true);
 }
 
@@ -226,8 +228,8 @@ bool cTimer::Parse(const char *s)
   char *channelbuffer = NULL;
   char *daybuffer = NULL;
   char *filebuffer = NULL;
-  free(summary);
-  summary = NULL;
+  free(aux);
+  aux = NULL;
   //XXX Apparently sscanf() doesn't work correctly if the last %a argument
   //XXX results in an empty string (this first occured when the EIT gathering
   //XXX was put into a separate thread - don't know why this happens...
@@ -244,16 +246,16 @@ bool cTimer::Parse(const char *s)
      s = s2;
      }
   bool result = false;
-  if (8 <= sscanf(s, "%d :%a[^:]:%a[^:]:%d :%d :%d :%d :%a[^:\n]:%a[^\n]", &flags, &channelbuffer, &daybuffer, &start, &stop, &priority, &lifetime, &filebuffer, &summary)) {
-     if (summary && !*skipspace(summary)) {
-        free(summary);
-        summary = NULL;
+  if (8 <= sscanf(s, "%u :%a[^:]:%a[^:]:%d :%d :%d :%d :%a[^:\n]:%a[^\n]", &flags, &channelbuffer, &daybuffer, &start, &stop, &priority, &lifetime, &filebuffer, &aux)) {
+     ClrFlags(tfRecording);
+     if (aux && !*skipspace(aux)) {
+        free(aux);
+        aux = NULL;
         }
      //TODO add more plausibility checks
      result = ParseDay(daybuffer, day, weekdays);
      strn0cpy(file, filebuffer, MaxFileName);
      strreplace(file, '|', ':');
-     strreplace(summary, '|', '\n');
      if (isnumber(channelbuffer))
         channel = Channels.GetByNumber(atoi(channelbuffer));
      else
@@ -290,7 +292,7 @@ int cTimer::GetWDay(time_t t)
 {
   struct tm tm_r;
   int weekday = localtime_r(&t, &tm_r)->tm_wday;
-  return weekday == 0 ? 6 : weekday - 1; // we start with monday==0!
+  return weekday == 0 ? 6 : weekday - 1; // we start with Monday==0!
 }
 
 bool cTimer::DayMatches(time_t t) const
@@ -328,7 +330,7 @@ char *cTimer::SetFile(const char *File)
   return file;
 }
 
-bool cTimer::Matches(time_t t, bool Directly) const
+bool cTimer::Matches(time_t t, bool Directly, int Margin) const
 {
   startTime = stopTime = 0;
   if (t == 0)
@@ -363,12 +365,12 @@ bool cTimer::Matches(time_t t, bool Directly) const
      }
 
   if (HasFlags(tfActive)) {
-     if (HasFlags(tfVps) && !Directly && event && event->Vps() && schedule && schedule->PresentSeenWithin(30)) {
+     if (HasFlags(tfVps) && !Directly && event && event->Vps() && event->Schedule() && event->Schedule()->PresentSeenWithin(30)) {
         startTime = event->StartTime();
         stopTime = event->EndTime();
         return event->IsRunning(true);
         }
-     return startTime <= t && t < stopTime; // must stop *before* stopTime to allow adjacent timers
+     return startTime <= t + Margin && t < stopTime; // must stop *before* stopTime to allow adjacent timers
      }
   return false;
 }
@@ -399,7 +401,9 @@ int cTimer::Matches(const cEvent *Event, int *Overlap) const
      startTime = stopTime = 0;
      if (Overlap)
         *Overlap = overlap;
-     return overlap >= 1000 ? tmFull : overlap > 0 ? tmPartial : tmNone;
+     if (UseVps)
+        return overlap > FULLMATCH ? tmFull : tmNone;
+     return overlap >= FULLMATCH ? tmFull : overlap > 0 ? tmPartial : tmNone;
      }
   return tmNone;
 }
@@ -408,7 +412,13 @@ int cTimer::Matches(const cEvent *Event, int *Overlap) const
 
 bool cTimer::Expired(void) const
 {
-  return IsSingleEvent() && !Recording() && StopTime() + EXPIRELATENCY <= time(NULL);
+  if (IsSingleEvent() && !Recording() && StopTime() + EXPIRELATENCY <= time(NULL)) {
+     if (HasFlags(tfVps) && event && event->Vps())
+        return event->RunningStatus() == SI::RunningStatusNotRunning;
+     else
+        return true;
+     }
+  return false;
 }
 
 time_t cTimer::StartTime(void) const
@@ -425,18 +435,74 @@ time_t cTimer::StopTime(void) const
   return stopTime;
 }
 
-void cTimer::SetEvent(const cSchedule *Schedule, const cEvent *Event)
+#define EPGLIMITBEFORE   (1 * 3600) // Time in seconds before a timer's start time and
+#define EPGLIMITAFTER    (1 * 3600) // after its stop time within which EPG events will be taken into consideration.
+
+void cTimer::SetEventFromSchedule(const cSchedules *Schedules)
+{
+  cSchedulesLock SchedulesLock;
+  if (!Schedules) {
+     lastSetEvent = 0; // forces setting the event, even if the schedule hasn't been modified
+     if (!(Schedules = cSchedules::Schedules(SchedulesLock)))
+        return;
+     }
+  const cSchedule *Schedule = Schedules->GetSchedule(Channel());
+  if (Schedule && Schedule->Events()->First()) {
+     time_t now = time(NULL);
+     if (!lastSetEvent || Schedule->Modified() >= lastSetEvent) {
+        lastSetEvent = now;
+        const cEvent *Event = NULL;
+        if (HasFlags(tfVps) && Schedule->Events()->First()->Vps()) {
+           if (event && Recording())
+              return; // let the recording end first
+           // VPS timers only match if their start time exactly matches the event's VPS time:
+           for (const cEvent *e = Schedule->Events()->First(); e; e = Schedule->Events()->Next(e)) {
+               if (e->StartTime() && e->RunningStatus() != SI::RunningStatusNotRunning) { // skip outdated events
+                  int overlap = 0;
+                  Matches(e, &overlap);
+                  if (overlap > FULLMATCH) {
+                     Event = e;
+                     break; // take the first matching event
+                     }
+                  }
+               }
+           if (!Event && event && (now <= event->EndTime() || Matches(0, true)))
+              return; // stay with the old event until the timer has completely expired
+           }
+        else {
+           // Normal timers match the event they have the most overlap with:
+           int Overlap = 0;
+           // Set up the time frame within which to check events:
+           Matches(0, true);
+           time_t TimeFrameBegin = StartTime() - EPGLIMITBEFORE;
+           time_t TimeFrameEnd   = StopTime()  + EPGLIMITAFTER;
+           for (const cEvent *e = Schedule->Events()->First(); e; e = Schedule->Events()->Next(e)) {
+               if (e->EndTime() < TimeFrameBegin)
+                  continue; // skip events way before the timer starts
+               if (e->StartTime() > TimeFrameEnd)
+                  break; // the rest is way after the timer ends
+               int overlap = 0;
+               Matches(e, &overlap);
+               if (overlap && overlap >= Overlap) {
+                  if (Event && overlap == Overlap && e->Duration() <= Event->Duration())
+                     continue; // if overlap is the same, we take the longer event
+                  Overlap = overlap;
+                  Event = e;
+                  }
+               }
+           }
+        SetEvent(Event);
+        }
+     }
+}
+
+void cTimer::SetEvent(const cEvent *Event)
 {
   if (event != Event) { //XXX TODO check event data, too???
-     if (Event) {
-        char vpsbuf[64] = "";
-        if (Event->Vps())
-           sprintf(vpsbuf, "(VPS: %s) ", *Event->GetVpsString());
-        isyslog("timer %s set to event %s %s-%s %s'%s'", *ToDescr(), *Event->GetDateString(), *Event->GetTimeString(), *Event->GetEndTimeString(), vpsbuf, Event->Title());
-        }
+     if (Event)
+        isyslog("timer %s set to event %s", *ToDescr(), *Event->ToDescr());
      else
         isyslog("timer %s set to no event", *ToDescr());
-     schedule = Event ? Schedule : NULL;
      event = Event;
      }
 }
@@ -463,22 +529,27 @@ void cTimer::SetInVpsMargin(bool InVpsMargin)
   inVpsMargin = InVpsMargin;
 }
 
-void cTimer::SetFlags(int Flags)
+void cTimer::SetPriority(int Priority)
+{
+  priority = Priority;
+}
+
+void cTimer::SetFlags(uint Flags)
 {
   flags |= Flags;
 }
 
-void cTimer::ClrFlags(int Flags)
+void cTimer::ClrFlags(uint Flags)
 {
   flags &= ~Flags;
 }
 
-void cTimer::InvFlags(int Flags)
+void cTimer::InvFlags(uint Flags)
 {
   flags ^= Flags;
 }
 
-bool cTimer::HasFlags(int Flags) const
+bool cTimer::HasFlags(uint Flags) const
 {
   return (flags & Flags) == Flags;
 }
@@ -486,7 +557,7 @@ bool cTimer::HasFlags(int Flags) const
 void cTimer::Skip(void)
 {
   day = IncDay(SetTime(StartTime(), 0), 1);
-  event = NULL;
+  SetEvent(NULL);
 }
 
 void cTimer::OnOff(void)
@@ -501,7 +572,7 @@ void cTimer::OnOff(void)
      Skip();
   else
      SetFlags(tfActive);
-  event = NULL;
+  SetEvent(NULL);
   Matches(); // refresh start and end time
 }
 
@@ -511,9 +582,10 @@ cTimers Timers;
 
 cTimers::cTimers(void)
 {
-  modified = false;
+  state = 0;
   beingEdited = 0;;
   lastSetEvents = 0;
+  lastDeleteExpired = 0;
 }
 
 cTimer *cTimers::GetTimer(cTimer *Timer)
@@ -530,13 +602,22 @@ cTimer *cTimers::GetTimer(cTimer *Timer)
 
 cTimer *cTimers::GetMatch(time_t t)
 {
+  static int LastPending = -1;
   cTimer *t0 = NULL;
   for (cTimer *ti = First(); ti; ti = Next(ti)) {
       if (!ti->Recording() && ti->Matches(t)) {
+         if (ti->Pending()) {
+            if (ti->Index() > LastPending)
+               LastPending = ti->Index();
+            else
+               continue;
+            }
          if (!t0 || ti->Priority() > t0->Priority())
             t0 = ti;
          }
       }
+  if (!t0)
+     LastPending = -1;
   return t0;
 }
 
@@ -562,7 +643,7 @@ cTimer *cTimers::GetNextActiveTimer(void)
 {
   cTimer *t0 = NULL;
   for (cTimer *ti = First(); ti; ti = Next(ti)) {
-      if ((ti->HasFlags(tfActive)) && (!t0 || ti->Compare(*t0) < 0))
+      if ((ti->HasFlags(tfActive)) && (!t0 || ti->StopTime() > time(NULL) && ti->Compare(*t0) < 0))
          t0 = ti;
       }
   return t0;
@@ -570,19 +651,16 @@ cTimer *cTimers::GetNextActiveTimer(void)
 
 void cTimers::SetModified(void)
 {
-  modified = true;
+  state++;
 }
 
-bool cTimers::Modified(void)
+bool cTimers::Modified(int &State)
 {
-  bool Result = modified;
-  modified = false;
+  bool Result = state != State;
+  State = state;
   return Result;
 }
 
-#define EPGLIMITPAST   (2 * 3600) // time in seconds around now, within which EPG events will be taken into consideration
-#define EPGLIMITFUTURE (4 * 3600)
-
 void cTimers::SetEvents(void)
 {
   if (time(NULL) - lastSetEvents < 5)
@@ -592,46 +670,9 @@ void cTimers::SetEvents(void)
   if (Schedules) {
      if (!lastSetEvents || Schedules->Modified() >= lastSetEvents) {
         for (cTimer *ti = First(); ti; ti = Next(ti)) {
-            const cSchedule *Schedule = Schedules->GetSchedule(ti->Channel()->GetChannelID());
-            if (Schedule) {
-               if (!lastSetEvents || Schedule->Modified() >= lastSetEvents) {
-                  const cEvent *Event = NULL;
-                  int Overlap = 0;
-                  int Distance = INT_MIN;
-                  time_t now = time(NULL);
-                  for (const cEvent *e = Schedule->Events()->First(); e; e = Schedule->Events()->Next(e)) {
-                      if (cRemote::HasKeys())
-                         return; // react immediately on user input
-                      if (e->EndTime() < now - EPGLIMITPAST)
-                         continue; // skip old events
-                      if (e->StartTime() > now + EPGLIMITFUTURE)
-                         break; // no need to process events too far in the future
-                      int overlap = 0;
-                      ti->Matches(e, &overlap);
-                      if (overlap && overlap >= Overlap) {
-                         int distance = 0;
-                         if (now < e->StartTime())
-                            distance = e->StartTime() - now;
-                         else if (now > e->EndTime())
-                            distance = e->EndTime() - now;
-                         if (Event && overlap == Overlap) {
-                            if (Overlap > FULLMATCH) { // this means VPS
-                               if (abs(Distance) < abs(distance)) 
-                                  break; // we've already found the closest VPS event
-                               }
-                            else if (e->Duration() <= Event->Duration())
-                               continue; // if overlap is the same, we take the longer event
-                            }
-                         Overlap = overlap;
-                         Distance = distance;
-                         Event = e;
-                         }
-                      }
-                  if (Event && Event->EndTime() < now - EXPIRELATENCY && !Event->IsRunning())
-                     Event = NULL;
-                  ti->SetEvent(Schedule, Event);
-                  }
-               }
+            if (cRemote::HasKeys())
+               return; // react immediately on user input
+            ti->SetEventFromSchedule(Schedules);
             }
         }
      }
@@ -640,6 +681,8 @@ void cTimers::SetEvents(void)
 
 void cTimers::DeleteExpired(void)
 {
+  if (time(NULL) - lastDeleteExpired < 30)
+     return;
   cTimer *ti = First();
   while (ti) {
         cTimer *next = Next(ti);
@@ -650,4 +693,5 @@ void cTimers::DeleteExpired(void)
            }
         ti = next;
         }
+  lastDeleteExpired = time(NULL);
 }
