@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.h 1.19 2005/05/07 10:36:35 kls Exp $
+ * $Id: timers.h 1.28 2006/04/08 12:41:44 kls Exp $
  */
 
 #ifndef __TIMERS_H
@@ -28,8 +28,9 @@ class cTimer : public cListObject {
   friend class cMenuEditTimer;
 private:
   mutable time_t startTime, stopTime;
+  time_t lastSetEvent;
   bool recording, pending, inVpsMargin;
-  int flags;
+  uint flags;
   cChannel *channel;
   mutable time_t day; ///< midnight of the day this timer shall hit, or of the first day it shall hit in case of a repeating timer
   int weekdays;       ///< bitmask, lowest bits: SSFTWTM  (the 'M' is the LSB)
@@ -38,11 +39,10 @@ private:
   int priority;
   int lifetime;
   char file[MaxFileName];
-  char *summary;
-  const cSchedule *schedule;
+  char *aux;
   const cEvent *event;
 public:
-  cTimer(bool Instant = false, bool Pause = false);
+  cTimer(bool Instant = false, bool Pause = false, cChannel *Channel = NULL);
   cTimer(const cEvent *Event);
   virtual ~cTimer();
   cTimer& operator= (const cTimer &Timer);
@@ -50,7 +50,7 @@ public:
   bool Recording(void) const { return recording; }
   bool Pending(void) const { return pending; }
   bool InVpsMargin(void) const { return inVpsMargin; }
-  int Flags(void) const { return flags; }
+  uint Flags(void) const { return flags; }
   const cChannel *Channel(void) const { return channel; }
   time_t Day(void) const { return day; }
   int WeekDays(void) const { return weekdays; }
@@ -60,7 +60,7 @@ public:
   int Lifetime(void) const { return lifetime; }
   const char *File(void) const { return file; }
   time_t FirstDay(void) const { return weekdays ? day : 0; }
-  const char *Summary(void) const { return summary; }
+  const char *Aux(void) const { return aux; }
   cString ToText(bool UseChannelID = false);
   cString ToDescr(void) const;
   const cEvent *Event(void) const { return event; }
@@ -73,19 +73,21 @@ public:
   static time_t IncDay(time_t t, int Days);
   static time_t SetTime(time_t t, int SecondsFromMidnight);
   char *SetFile(const char *File);
-  bool Matches(time_t t = 0, bool Directly = false) const;
+  bool Matches(time_t t = 0, bool Directly = false, int Margin = 0) const;
   int Matches(const cEvent *Event, int *Overlap = NULL) const;
   bool Expired(void) const;
   time_t StartTime(void) const;
   time_t StopTime(void) const;
-  void SetEvent(const cSchedule *Schedule, const cEvent *Event);
+  void SetEventFromSchedule(const cSchedules *Schedules = NULL);
+  void SetEvent(const cEvent *Event);
   void SetRecording(bool Recording);
   void SetPending(bool Pending);
   void SetInVpsMargin(bool InVpsMargin);
-  void SetFlags(int Flags);
-  void ClrFlags(int Flags);
-  void InvFlags(int Flags);
-  bool HasFlags(int Flags) const;
+  void SetPriority(int Priority);
+  void SetFlags(uint Flags);
+  void ClrFlags(uint Flags);
+  void InvFlags(uint Flags);
+  bool HasFlags(uint Flags) const;
   void Skip(void);
   void OnOff(void);
   cString PrintFirstDay(void) const;
@@ -96,9 +98,10 @@ public:
 
 class cTimers : public cConfig<cTimer> {
 private:
-  bool modified;
+  int state;
   int beingEdited;
   time_t lastSetEvents;
+  time_t lastDeleteExpired;
 public:
   cTimers(void);
   cTimer *GetTimer(cTimer *Timer);
@@ -109,9 +112,10 @@ public:
   void IncBeingEdited(void) { beingEdited++; }
   void DecBeingEdited(void) { if (!--beingEdited) lastSetEvents = 0; }
   void SetModified(void);
-  bool Modified(void);
-      ///< Returns true if any of the timers have been modified.
-      ///< Calling this function resets the 'modified' flag to false.
+  bool Modified(int &State);
+      ///< Returns true if any of the timers have been modified, which
+      ///< is detected by State being different than the internal state.
+      ///< Upon return the internal state will be stored in State.
   void SetEvents(void);
   void DeleteExpired(void);
   };
