@@ -1,7 +1,7 @@
 /*
  * vdr.c: Video Disk Recorder main program
  *
- * Copyright (C) 2000, 2003 Klaus Schmidinger
+ * Copyright (C) 2000, 2003, 2006 Klaus Schmidinger
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,20 +15,24 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
  *
  * The author can be reached at kls@cadsoft.de
  *
  * The project's page is at http://www.cadsoft.de/vdr
  *
- * $Id: vdr.c 1.220 2005/11/27 15:56:18 kls Exp $
+ * $Id: vdr.c 1.267 2006/04/29 09:14:06 kls Exp $
  */
 
 #include <getopt.h>
+#include <grp.h>
 #include <locale.h>
+#include <pwd.h>
 #include <signal.h>
 #include <stdlib.h>
+#include <sys/capability.h>
+#include <sys/prctl.h>
 #include <termios.h>
 #include <unistd.h>
 #include "audio.h"
@@ -67,11 +71,69 @@
                               // in order to react on a possible new CAM menu as soon as possible
 #define DEVICEREADYTIMEOUT 30 // seconds to wait until all devices are ready
 #define MENUTIMEOUT       120 // seconds of user inactivity after which an OSD display is closed
+#define SHUTDOWNRETRY     300 // seconds before trying again to shut down
+#define VPSCHECKDELTA      10 // seconds between checks for timers that have entered the VPS margin
+#define VPSDEVICETIMEOUT    8 // seconds before a device used for VPS may be reused
 
 #define EXIT(v) { ExitCode = (v); goto Exit; }
 
 static int Interrupted = 0;
 
+static bool SetUser(const char *UserName)
+{
+  if (UserName) {
+     struct passwd *user = getpwnam(UserName);
+     if (!user) {
+        fprintf(stderr, "vdr: unknown user: '%s'\n", UserName);
+        return false;
+        }
+     if (setgid(user->pw_gid) < 0) {
+        fprintf(stderr, "vdr: cannot set group id %u: %s\n", (unsigned int)user->pw_gid, strerror(errno));
+        return false;
+        }
+     if (initgroups(user->pw_name, user->pw_gid) < 0) {
+        fprintf(stderr, "vdr: cannot set supplemental group ids for user %s: %s\n", user->pw_name, strerror(errno));
+        return false;
+        }
+     if (setuid(user->pw_uid) < 0) {
+        fprintf(stderr, "vdr: cannot set user id %u: %s\n", (unsigned int)user->pw_uid, strerror(errno));
+        return false;
+        }
+     if (prctl(PR_SET_DUMPABLE, 2, 0, 0, 0) < 0) {
+        fprintf(stderr, "vdr: warning - cannot set dumpable: %s\n", strerror(errno));
+        // always non-fatal, and will not work with kernel < 2.6.13
+        }
+     }
+  return true;
+}
+
+static bool SetCapSysTime(void)
+{
+  // drop all capabilities except cap_sys_time
+  cap_t caps = cap_from_text("= cap_sys_time=ep");
+  if (!caps) {
+     fprintf(stderr, "vdr: cap_from_text failed: %s\n", strerror(errno));
+     return false;
+     }
+  if (cap_set_proc(caps) == -1) {
+     fprintf(stderr, "vdr: cap_set_proc failed: %s\n", strerror(errno));
+     cap_free(caps);
+     return false;
+     }
+  cap_free(caps);
+  return true;
+}
+
+static bool SetKeepCaps(bool On)
+{
+  // set keeping capabilities during setuid() on/off
+  if (prctl(PR_SET_KEEPCAPS, On ? 1 : 0, 0, 0, 0) != 0) {
+     fprintf(stderr, "vdr: prctl failed\n");
+     return false;
+     }
+  return true;
+}
+
 static void SignalHandler(int signum)
 {
   if (signum != SIGPIPE) {
@@ -107,6 +169,8 @@ int main(int argc, char *argv[])
 #define DEFAULTPLUGINDIR PLUGINDIR
 #define DEFAULTEPGDATAFILENAME "epg.data"
 
+  bool StartedAsRoot = false;
+  const char *VdrUser = NULL;
   int SVDRPport = DEFAULTSVDRPPORT;
   const char *AudioCommand = NULL;
   const char *ConfigDirectory = NULL;
@@ -134,6 +198,9 @@ int main(int argc, char *argv[])
 #if defined(VFAT)
   VfatFileSystem = true;
 #endif
+#if defined(VDR_USER)
+  VdrUser = VDR_USER;
+#endif
 
   cPluginManager PluginManager(DEFAULTPLUGINDIR);
   int ExitCode = 0;
@@ -144,6 +211,7 @@ int main(int argc, char *argv[])
       { "daemon",   no_argument,       NULL, 'd' },
       { "device",   required_argument, NULL, 'D' },
       { "epgfile",  required_argument, NULL, 'E' },
+      { "grab",     required_argument, NULL, 'g' },
       { "help",     no_argument,       NULL, 'h' },
       { "lib",      required_argument, NULL, 'L' },
       { "lirc",     optional_argument, NULL, 'l' | 0x100 },
@@ -156,6 +224,7 @@ int main(int argc, char *argv[])
       { "record",   required_argument, NULL, 'r' },
       { "shutdown", required_argument, NULL, 's' },
       { "terminal", required_argument, NULL, 't' },
+      { "user",     required_argument, NULL, 'u' },
       { "version",  no_argument,       NULL, 'V' },
       { "vfat",     no_argument,       NULL, 'v' | 0x100 },
       { "video",    required_argument, NULL, 'v' },
@@ -164,7 +233,7 @@ int main(int argc, char *argv[])
     };
 
   int c;
-  while ((c = getopt_long(argc, argv, "a:c:dD:E:hl:L:mp:P:r:s:t:v:Vw:", long_options, NULL)) != -1) {
+  while ((c = getopt_long(argc, argv, "a:c:dD:E:g:hl:L:mp:P:r:s:t:u:v:Vw:", long_options, NULL)) != -1) {
         switch (c) {
           case 'a': AudioCommand = optarg;
                     break;
@@ -183,6 +252,8 @@ int main(int argc, char *argv[])
                     break;
           case 'E': EpgDataFileName = (*optarg != '-' ? optarg : NULL);
                     break;
+          case 'g': cSVDRP::SetGrabImageDir(*optarg != '-' ? optarg : NULL);
+                    break;
           case 'h': DisplayHelp = true;
                     break;
           case 'l': {
@@ -248,6 +319,9 @@ int main(int argc, char *argv[])
                        return 2;
                        }
                     break;
+          case 'u': if (*optarg)
+                       VdrUser = optarg;
+                    break;
           case 'V': DisplayVersion = true;
                     break;
           case 'v' | 0x100:
@@ -270,6 +344,22 @@ int main(int argc, char *argv[])
           }
         }
 
+  // Set user id in case we were started as root:
+
+  if (VdrUser && geteuid() == 0) {
+     StartedAsRoot = true;
+     if (strcmp(VdrUser, "root")) {
+        if (!SetKeepCaps(true))
+           return 2;
+        if (!SetUser(VdrUser))
+           return 2;
+        if (!SetKeepCaps(false))
+           return 2;
+        if (!SetCapSysTime())
+           return 2;
+        }
+     }
+
   // Help and version info:
 
   if (DisplayHelp || DisplayVersion) {
@@ -285,11 +375,15 @@ int main(int argc, char *argv[])
                "  -D NUM,   --device=NUM   use only the given DVB device (NUM = 0, 1, 2...)\n"
                "                           there may be several -D options (default: all DVB\n"
                "                           devices will be used)\n"
-               "  -E FILE   --epgfile=FILE write the EPG data into the given FILE (default is\n"
+               "  -E FILE,  --epgfile=FILE write the EPG data into the given FILE (default is\n"
                "                           '%s' in the video directory)\n"
                "                           '-E-' disables this\n"
                "                           if FILE is a directory, the default EPG file will be\n"
                "                           created in that directory\n"
+               "  -g DIR,   --grab=DIR     write images from the SVDRP command GRAB into the\n"
+               "                           given DIR; DIR must be the full path name of an\n"
+               "                           existing directory, without any \"..\", double '/'\n"
+               "                           or symlinks (default: none, same as -g-)\n"
                "  -h,       --help         print this help and exit\n"
                "  -l LEVEL, --log=LEVEL    set log level (default: 3)\n"
                "                           0 = no logging, 1 = errors only,\n"
@@ -309,6 +403,8 @@ int main(int argc, char *argv[])
                "  -r CMD,   --record=CMD   call CMD before and after a recording\n"
                "  -s CMD,   --shutdown=CMD call CMD to shutdown the computer\n"
                "  -t TTY,   --terminal=TTY controlling tty\n"
+               "  -u USER,  --user=USER    run as user USER; only applicable if started as\n"
+               "                           root\n"
                "  -v DIR,   --video=DIR    use DIR as video directory (default: %s)\n"
                "  -V,       --version      print version information and exit\n"
                "            --vfat         encode special characters in recording names to\n"
@@ -326,7 +422,7 @@ int main(int argc, char *argv[])
                );
         }
      if (DisplayVersion)
-        printf("vdr (%s) - The Video Disk Recorder\n", VDRVERSION);
+        printf("vdr (%s/%s) - The Video Disk Recorder\n", VDRVERSION, APIVERSION);
      if (PluginManager.HasPlugins()) {
         if (DisplayHelp)
            printf("Plugins: vdr -P\"name [OPTIONS]\"\n\n");
@@ -358,7 +454,7 @@ int main(int argc, char *argv[])
   // Log file:
 
   if (SysLogLevel > 0)
-     openlog("vdr", LOG_PID | LOG_CONS, SysLogTarget);
+     openlog("vdr", LOG_CONS, SysLogTarget); // LOG_PID doesn't work as expected under NPTL
 
   // Check the video directory:
 
@@ -371,7 +467,7 @@ int main(int argc, char *argv[])
 
   if (DaemonMode) {
      if (daemon(1, 0) == -1) {
-        fprintf(stderr, "%m\n");
+        fprintf(stderr, "vdr: %m\n");
         esyslog("ERROR: %m");
         return 2;
         }
@@ -385,12 +481,16 @@ int main(int argc, char *argv[])
      }
 
   isyslog("VDR version %s started", VDRVERSION);
+  if (StartedAsRoot && VdrUser)
+     isyslog("switched to user '%s'", VdrUser);
+  if (DaemonMode)
+     dsyslog("running as daemon (tid=%d)", cThread::ThreadId());
+  cThread::SetMainThreadId();
 
   // Main program loop variables - need to be here to have them initialized before any EXIT():
 
   cOsdObject *Menu = NULL;
-  cOsdObject *Temp = NULL;
-  int LastChannel = -1;
+  int LastChannel = 0;
   int LastTimerChannel = -1;
   int PreviousChannel[2] = { 1, 1 };
   int PreviousChannelIndex = 0;
@@ -401,6 +501,7 @@ int main(int argc, char *argv[])
   bool ForceShutdown = false;
   bool UserShutdown = false;
   bool TimerInVpsMargin = false;
+  bool IsInfoMenu = false;
 
   // Load plugins:
 
@@ -423,7 +524,6 @@ int main(int argc, char *argv[])
         Commands.Load(AddDirectory(ConfigDirectory, "commands.conf"), true) &&
         RecordingCommands.Load(AddDirectory(ConfigDirectory, "reccmds.conf"), true) &&
         SVDRPhosts.Load(AddDirectory(ConfigDirectory, "svdrphosts.conf"), true) &&
-        CaDefinitions.Load(AddDirectory(ConfigDirectory, "ca.conf"), true) &&
         Keys.Load(AddDirectory(ConfigDirectory, "remote.conf")) &&
         KeyMacros.Load(AddDirectory(ConfigDirectory, "keymacros.conf"), true)
         ))
@@ -434,6 +534,7 @@ int main(int argc, char *argv[])
   // Recordings:
 
   Recordings.Update();
+  DeletedRecordings.Update();
 
   // EPG data:
 
@@ -531,6 +632,10 @@ int main(int argc, char *argv[])
 
   if (!cDevice::WaitForAllDevicesReady(DEVICEREADYTIMEOUT))
      dsyslog("not all devices ready after %d seconds", DEVICEREADYTIMEOUT);
+  if (Setup.InitialChannel > 0)
+     Setup.CurrentChannel = Setup.InitialChannel;
+  if (Setup.InitialVolume >= 0)
+     Setup.CurrentVolume = Setup.InitialVolume;
   Channels.SwitchTo(Setup.CurrentChannel);
   if (MuteAudio)
      cDevice::PrimaryDevice()->ToggleMute();
@@ -555,6 +660,8 @@ int main(int argc, char *argv[])
 
   // Main program loop:
 
+#define DELETE_MENU ((IsInfoMenu &= (Menu == NULL)), delete Menu, Menu = NULL)
+
   while (!Interrupted) {
         // Handle emergency exits:
         if (cThread::EmergencyExit()) {
@@ -576,8 +683,8 @@ int main(int argc, char *argv[])
                      && !(LastTimerChannel > 0 && Channels.SwitchTo(LastTimerChannel)) // ...or the one used by the last timer...
                      && !cDevice::SwitchChannel(1) // ...or the next higher available one...
                      && !cDevice::SwitchChannel(-1)) // ...or the next lower available one
-                    ; 
-                 }   
+                    ;
+                 }
               lastTime = time(NULL); // don't do this too often
               LastTimerChannel = -1;
               }
@@ -594,9 +701,10 @@ int main(int argc, char *argv[])
         if (!Channels.BeingEdited() && !Timers.BeingEdited()) {
            int modified = Channels.Modified();
            static time_t ChannelSaveTimeout = 0;
+           static int TimerState = 0;
            // Channels and timers need to be stored in a consistent manner,
            // therefore if one of them is changed, we save both.
-           if (modified == CHANNELSMOD_USER || Timers.Modified())
+           if (modified == CHANNELSMOD_USER || Timers.Modified(TimerState))
               ChannelSaveTimeout = 1; // triggers an immediate save
            else if (modified && !ChannelSaveTimeout)
               ChannelSaveTimeout = time(NULL) + CHANNELSAVEDELTA;
@@ -611,7 +719,7 @@ int main(int argc, char *argv[])
                   if (Channel->Modification(CHANNELMOD_RETUNE)) {
                      cRecordControls::ChannelDataModified(Channel);
                      if (Channel->Number() == cDevice::CurrentChannel()) {
-                        if (!cDevice::PrimaryDevice()->Replaying() || cTransferControl::ReceiverDevice()) {
+                        if (!cDevice::PrimaryDevice()->Replaying() || cDevice::PrimaryDevice()->Transferring()) {
                            if (cDevice::ActualDevice()->ProvidesTransponder(Channel)) { // avoids retune on devices that don't really access the transponder
                               isyslog("retuning due to modification of channel %d", Channel->Number());
                               Channels.SwitchTo(Channel->Number());
@@ -626,16 +734,14 @@ int main(int argc, char *argv[])
         // Channel display:
         if (!EITScanner.Active() && cDevice::CurrentChannel() != LastChannel) {
            if (!Menu)
-              Menu = Temp = new cDisplayChannel(cDevice::CurrentChannel(), LastChannel > 0);
+              Menu = new cDisplayChannel(cDevice::CurrentChannel(), LastChannel >= 0);
            LastChannel = cDevice::CurrentChannel();
            LastChannelChanged = time(NULL);
            }
-        if (time(NULL) - LastChannelChanged >= Setup.ZapTimeout && LastChannel != PreviousChannel[0] && LastChannel != PreviousChannel[1])
+        if (time(NULL) - LastChannelChanged >= Setup.ZapTimeout && LastChannel != PreviousChannel[PreviousChannelIndex])
            PreviousChannel[PreviousChannelIndex ^= 1] = LastChannel;
         // Timers and Recordings:
         if (!Timers.BeingEdited()) {
-           // Delete expired timers:
-           Timers.DeleteExpired();
            // Assign events to timers:
            Timers.SetEvents();
            // Must do all following calls with the exact same time!
@@ -651,24 +757,68 @@ int main(int argc, char *argv[])
                  LastTimerChannel = Timer->Channel()->Number();
               }
            // Make sure VPS timers "see" their channel early enough:
-           TimerInVpsMargin = false;
-           for (cTimer *Timer = Timers.First(); Timer; Timer = Timers.Next(Timer)) {
-               if (Timer->HasFlags(tfActive | tfVps) && !Timer->Recording() && !Timer->Pending() && Timer->Matches(Now + Setup.VpsMargin, true)) {
-                  if (!Timer->InVpsMargin()) {
+           static time_t LastVpsCheck = 0;
+           if (Now - LastVpsCheck > VPSCHECKDELTA) { // don't do this too often
+              TimerInVpsMargin = false;
+              static time_t DeviceUsed[MAXDEVICES] = { 0 };
+              for (cTimer *Timer = Timers.First(); Timer; Timer = Timers.Next(Timer)) {
+                  if (Timer->HasFlags(tfActive | tfVps) && !Timer->Recording() && Timer->Matches(Now, true, Setup.VpsMargin)) {
                      Timer->SetInVpsMargin(true);
-                     //XXX if not primary device has TP???
-                     LastTimerChannel = Timer->Channel()->Number();
-                     cRecordControls::Start(Timer); // will only switch the device
+                     // Find a device that provides the required transponder:
+                     cDevice *Device = NULL;
+                     for (int i = 0; i < cDevice::NumDevices(); i++) {
+                         cDevice *d = cDevice::GetDevice(i);
+                         if (d && d->ProvidesTransponder(Timer->Channel())) {
+                            if (d->IsTunedToTransponder(Timer->Channel())) {
+                               // if any device is tuned to the transponder, we're done
+                               Device = d;
+                               break;
+                               }
+                            else if (Now - DeviceUsed[d->DeviceNumber()] > VPSDEVICETIMEOUT) {
+                               // only check other devices if they have been left alone for a while
+                               if (d->MaySwitchTransponder())
+                                  // this one can be switched without disturbing anything else
+                                  Device = d;
+                               else if (!Device && !d->Receiving() && d->ProvidesTransponderExclusively(Timer->Channel()))
+                                  // use this one only if no other with less impact can be found
+                                  Device = d;
+                               }
+                            }
+                         }
+                     if (!Device) {
+                        cDevice *d = cDevice::ActualDevice();
+                        if (!d->Receiving() && d->ProvidesTransponder(Timer->Channel()) && Now - DeviceUsed[d->DeviceNumber()] > VPSDEVICETIMEOUT)
+                           Device = d; // use the actual device as a last resort
+                        }
+                     // Switch the device to the transponder:
+                     if (Device) {
+                        if (!Device->IsTunedToTransponder(Timer->Channel())) {
+                           if (Device == cDevice::ActualDevice() && !Device->IsPrimaryDevice())
+                              cDevice::PrimaryDevice()->StopReplay(); // stop transfer mode
+                           dsyslog("switching device %d to channel %d", Device->DeviceNumber() + 1, Timer->Channel()->Number());
+                           Device->SwitchChannel(Timer->Channel(), false);
+                           DeviceUsed[Device->DeviceNumber()] = Now;
+                           }
+                        if (cDevice::PrimaryDevice()->HasDecoder() && !cDevice::PrimaryDevice()->HasProgramme()) {
+                           // the previous SwitchChannel() has switched away the current live channel
+                           Channels.SwitchTo(Timer->Channel()->Number()); // avoids toggling between old channel and black screen
+                           Skins.Message(mtInfo, tr("Upcoming VPS recording!"));
+                           }
+                        }
+                     TimerInVpsMargin = true;
                      }
+                  else
+                     Timer->SetInVpsMargin(false);
                   }
-               else
-                  Timer->SetInVpsMargin(false);
-               if (Timer->InVpsMargin())
-                  TimerInVpsMargin = true;
-               }
+              LastVpsCheck = time(NULL);
+              }
+           // Delete expired timers:
+           Timers.DeleteExpired();
            }
-        if (!Menu && Recordings.NeedsUpdate())
+        if (!Menu && Recordings.NeedsUpdate()) {
            Recordings.Update();
+           DeletedRecordings.Update();
+           }
         // CAM control:
         if (!Menu && !cOsd::IsOpen()) {
            Menu = CamControl();
@@ -690,24 +840,49 @@ int main(int argc, char *argv[])
         // Keys that must work independent of any interactive mode:
         switch (key) {
           // Menu control:
-          case kMenu:
+          case kMenu: {
                key = kNone; // nobody else needs to see this key
-               if (Menu) {
-                  DELETENULL(Menu);
-                  if (!Temp)
-                     break;
+               bool WasOpen = Interact != NULL;
+               bool WasMenu = Interact && Interact->IsMenu();
+               if (Menu)
+                  DELETE_MENU;
+               else if (cControl::Control()) {
+                  if (cOsd::IsOpen())
+                     cControl::Control()->Hide();
+                  else
+                     WasOpen = false;
                   }
-               if (cControl::Control())
-                  cControl::Control()->Hide();
-               Menu = new cMenuMain(cControl::Control());
-               Temp = NULL;
+               if (!WasOpen || !WasMenu && !Setup.MenuButtonCloses)
+                  Menu = new cMenuMain;
+               }
                break;
+          // Info:
+          case kInfo: {
+               bool WasInfoMenu = IsInfoMenu;
+               DELETE_MENU;
+               if (!WasInfoMenu) {
+                  IsInfoMenu = true;
+                  if (cControl::Control()) {
+                     cControl::Control()->Hide();
+                     Menu = cControl::Control()->GetInfo();
+                     if (Menu)
+                        Menu->Show();
+                     else
+                        IsInfoMenu = false;
+                     }
+                  else {
+                     cRemote::Put(kOk, true);
+                     cRemote::Put(kSchedule, true);
+                     }
+                  }
+               }
+               break;
+          // Direct main menu functions:
           #define DirectMainFunction(function)\
-            DELETENULL(Menu);\
+            DELETE_MENU;\
             if (cControl::Control())\
                cControl::Control()->Hide();\
-            Menu = new cMenuMain(cControl::Control(), function);\
-            Temp = NULL;\
+            Menu = new cMenuMain(function);\
             key = kNone; // nobody else needs to see this key
           case kSchedule:   DirectMainFunction(osSchedule); break;
           case kChannels:   DirectMainFunction(osChannels); break;
@@ -717,18 +892,14 @@ int main(int argc, char *argv[])
           case kCommands:   DirectMainFunction(osCommands); break;
           case kUser1 ... kUser9: cRemote::PutMacro(key); key = kNone; break;
           case k_Plugin: {
-               DELETENULL(Menu);
-               Temp = NULL;
+               DELETE_MENU;
                if (cControl::Control())
                   cControl::Control()->Hide();
                cPlugin *plugin = cPluginManager::GetPlugin(cRemote::GetPlugin());
                if (plugin) {
-                  Menu = Temp = plugin->MainMenuAction();
-                  if (Menu) {
+                  Menu = plugin->MainMenuAction();
+                  if (Menu)
                      Menu->Show();
-                     if (Menu->IsMenu())
-                        ((cOsdMenu*)Menu)->Display();
-                     }
                   }
                else
                   esyslog("ERROR: unknown plugin '%s'", cRemote::GetPlugin());
@@ -740,7 +911,15 @@ int main(int argc, char *argv[])
           case kChanUp:
           case kChanDn|k_Repeat:
           case kChanDn:
-               cDevice::SwitchChannel(NORMALKEY(key) == kChanUp ? 1 : -1);
+               if (!Interact)
+                  Menu = new cDisplayChannel(NORMALKEY(key));
+               else if (cDisplayChannel::IsOpen()) {
+                  Interact->ProcessKey(key);
+                  continue;
+                  }
+               else
+                  cDevice::SwitchChannel(NORMALKEY(key) == kChanUp ? 1 : -1);
+               key = kNone; // nobody else needs to see these keys
                break;
           // Volume control:
           case kVolUp|k_Repeat:
@@ -757,7 +936,7 @@ int main(int argc, char *argv[])
                else
                   cDevice::PrimaryDevice()->SetVolume(NORMALKEY(key) == kVolDn ? -VOLUMEDELTA : VOLUMEDELTA);
                if (!Menu && !cOsd::IsOpen())
-                  Menu = Temp = cDisplayVolume::Create();
+                  Menu = cDisplayVolume::Create();
                cDisplayVolume::Process(key);
                key = kNone; // nobody else needs to see these keys
                break;
@@ -765,12 +944,10 @@ int main(int argc, char *argv[])
           case kAudio:
                if (cControl::Control())
                   cControl::Control()->Hide();
-               if (Temp && !cDisplayTracks::IsOpen()) {
-                  DELETENULL(Menu);
-                  Temp = NULL;
+               if (!cDisplayTracks::IsOpen()) {
+                  DELETE_MENU;
+                  Menu = cDisplayTracks::Create();
                   }
-               if (!Menu && !cOsd::IsOpen())
-                  Menu = Temp = cDisplayTracks::Create();
                else
                   cDisplayTracks::Process(key);
                key = kNone;
@@ -778,8 +955,7 @@ int main(int argc, char *argv[])
           // Pausing live video:
           case kPause:
                if (!cControl::Control()) {
-                  DELETENULL(Menu);
-                  Temp = NULL;
+                  DELETE_MENU;
                   if (!cRecordControls::PauseLiveVideo())
                      Skins.Message(mtError, tr("No free DVB device to record!"));
                   key = kNone; // nobody else needs to see this key
@@ -789,27 +965,28 @@ int main(int argc, char *argv[])
           case kRecord:
                if (!cControl::Control()) {
                   if (cRecordControls::Start())
-                     ;//XXX Skins.Message(mtInfo, tr("Recording"));
-                  else
-                     Skins.Message(mtError, tr("No free DVB device to record!"));
+                     Skins.Message(mtInfo, tr("Recording started"));
                   key = kNone; // nobody else needs to see this key
                   }
                break;
           // Power off:
-          case kPower: isyslog("Power button pressed");
-                       DELETENULL(Menu);
-                       Temp = NULL;
-                       if (!Shutdown) {
-                          Skins.Message(mtError, tr("Can't shutdown - option '-s' not given!"));
-                          break;
-                          }
-                       if (cRecordControls::Active()) {
-                          if (Interface->Confirm(tr("Recording - shut down anyway?")))
-                             ForceShutdown = true;
-                          }
-                       LastActivity = 1; // not 0, see below!
-                       UserShutdown = true;
-                       break;
+          case kPower:
+               isyslog("Power button pressed");
+               DELETE_MENU;
+               if (!Shutdown) {
+                  Skins.Message(mtError, tr("Can't shutdown - option '-s' not given!"));
+                  break;
+                  }
+               LastActivity = 1; // not 0, see below!
+               UserShutdown = true;
+               if (cRecordControls::Active()) {
+                  if (!Interface->Confirm(tr("Recording - shut down anyway?")))
+                     break;
+                  }
+               if (cPluginManager::Active(tr("shut down anyway?")))
+                  break;
+               ForceShutdown = true;
+               break;
           default: break;
           }
         Interact = Menu ? Menu : cControl::Control(); // might have been closed in the mean time
@@ -828,61 +1005,57 @@ int main(int argc, char *argv[])
                  state = osEnd;
               }
            switch (state) {
-             case osPause:  DELETENULL(Menu);
+             case osPause:  DELETE_MENU;
                             cControl::Shutdown(); // just in case
-                            Temp = NULL;
                             if (!cRecordControls::PauseLiveVideo())
                                Skins.Message(mtError, tr("No free DVB device to record!"));
                             break;
-             case osRecord: DELETENULL(Menu);
-                            Temp = NULL;
+             case osRecord: DELETE_MENU;
                             if (cRecordControls::Start())
-                               ;//XXX Skins.Message(mtInfo, tr("Recording"));
-                            else
-                               Skins.Message(mtError, tr("No free DVB device to record!"));
+                               Skins.Message(mtInfo, tr("Recording started"));
                             break;
              case osRecordings:
-                            DELETENULL(Menu);
+                            DELETE_MENU;
                             cControl::Shutdown();
-                            Temp = NULL;
-                            Menu = new cMenuMain(false, osRecordings);
+                            Menu = new cMenuMain(osRecordings);
                             break;
-             case osReplay: DELETENULL(Menu);
+             case osReplay: DELETE_MENU;
                             cControl::Shutdown();
-                            Temp = NULL;
                             cControl::Launch(new cReplayControl);
                             break;
              case osStopReplay:
-                            DELETENULL(Menu);
+                            DELETE_MENU;
                             cControl::Shutdown();
-                            Temp = NULL;
                             break;
              case osSwitchDvb:
-                            DELETENULL(Menu);
+                            DELETE_MENU;
                             cControl::Shutdown();
-                            Temp = NULL;
                             Skins.Message(mtInfo, tr("Switching primary DVB..."));
                             cDevice::SetPrimaryDevice(Setup.PrimaryDVB);
                             break;
-             case osPlugin: DELETENULL(Menu);
-                            Menu = Temp = cMenuMain::PluginOsdObject();
+             case osPlugin: DELETE_MENU;
+                            Menu = cMenuMain::PluginOsdObject();
                             if (Menu)
                                Menu->Show();
                             break;
              case osBack:
              case osEnd:    if (Interact == Menu)
-                               DELETENULL(Menu);
+                               DELETE_MENU;
                             else
                                cControl::Shutdown();
-                            Temp = NULL;
                             break;
              default:       ;
              }
            }
         else {
            // Key functions in "normal" viewing mode:
+           if (key != kNone && KeyMacros.Get(key)) {
+              cRemote::PutMacro(key);
+              key = kNone;
+              }
            switch (key) {
              // Toggle channels:
+             case kChanPrev:
              case k0: {
                   if (PreviousChannel[PreviousChannelIndex ^ 1] == LastChannel || LastChannel != PreviousChannel[0] && LastChannel != PreviousChannel[1])
                      PreviousChannelIndex ^= 1;
@@ -891,21 +1064,22 @@ int main(int argc, char *argv[])
                   }
              // Direct Channel Select:
              case k1 ... k9:
-                  Menu = new cDisplayChannel(key);
-                  break;
-             // Left/Right rotates trough channel groups:
+             // Left/Right rotates through channel groups:
              case kLeft|k_Repeat:
              case kLeft:
              case kRight|k_Repeat:
              case kRight:
-                  Menu = new cDisplayChannel(NORMALKEY(key));
-                  break;
+             // Previous/Next rotates through channel groups:
+             case kPrev|k_Repeat:
+             case kPrev:
+             case kNext|k_Repeat:
+             case kNext:
              // Up/Down Channel Select:
              case kUp|k_Repeat:
              case kUp:
              case kDown|k_Repeat:
              case kDown:
-                  cDevice::SwitchChannel(NORMALKEY(key) == kUp ? 1 : -1);
+                  Menu = new cDisplayChannel(NORMALKEY(key));
                   break;
              // Viewing Control:
              case kOk:   LastChannel = -1; break; // forces channel display
@@ -913,15 +1087,9 @@ int main(int argc, char *argv[])
              case kPlay:
                   if (cReplayControl::LastReplayed()) {
                      cControl::Shutdown();
-                     Temp = NULL;
                      cControl::Launch(new cReplayControl);
                      }
                   break;
-             // Key macros:
-             case kRed:
-             case kGreen:
-             case kYellow:
-             case kBlue: cRemote::PutMacro(key); break;
              default:    break;
              }
            }
@@ -935,7 +1103,7 @@ int main(int argc, char *argv[])
                  Skins.Message(mtInfo, tr("Editing process finished"));
               }
            }
-        if (!Interact && ((!cRecordControls::Active() && !cCutter::Active() && (!Interface->HasSVDRPConnection() || UserShutdown)) || ForceShutdown)) {
+        if (!Interact && ((!cRecordControls::Active() && !cCutter::Active() && !cPluginManager::Active() && (!Interface->HasSVDRPConnection() || UserShutdown)) || ForceShutdown)) {
            time_t Now = time(NULL);
            if (Now - LastActivity > ACTIVITYTIMEOUT) {
               // Shutdown:
@@ -955,7 +1123,7 @@ int main(int argc, char *argv[])
                     }
                  if (UserShutdown && Next && Delta <= Setup.MinEventTimeout * 60 && !ForceShutdown) {
                     char *buf;
-                    asprintf(&buf, tr("Recording in %d minutes, shut down anyway?"), Delta / 60);
+                    asprintf(&buf, tr("Recording in %ld minutes, shut down anyway?"), Delta / 60);
                     if (Interface->Confirm(buf))
                        ForceShutdown = true;
                     else
@@ -972,19 +1140,23 @@ int main(int argc, char *argv[])
                        cControl::Shutdown();
                        int Channel = timer ? timer->Channel()->Number() : 0;
                        const char *File = timer ? timer->File() : "";
-                       Delta = Next - time(NULL); // compensates for Confirm() timeout
+                       if (timer)
+                          Delta = Next - time(NULL); // compensates for Confirm() timeout
                        char *cmd;
                        asprintf(&cmd, "%s %ld %ld %d \"%s\" %d", Shutdown, Next, Delta, Channel, *strescape(File, "\"$"), UserShutdown);
                        isyslog("executing '%s'", cmd);
                        SystemExec(cmd);
                        free(cmd);
+                       LastActivity = time(NULL) - Setup.MinUserInactivity * 60 + SHUTDOWNRETRY; // try again later
                        }
-                    else if (WatchdogTimeout > 0) {
-                       alarm(WatchdogTimeout);
-                       if (signal(SIGALRM, Watchdog) == SIG_IGN)
-                          signal(SIGALRM, SIG_IGN);
-                       }
-                    LastActivity = time(NULL); // don't try again too soon
+                    else {
+                      LastActivity = Now;
+                      if (WatchdogTimeout > 0) {
+                         alarm(WatchdogTimeout);
+                         if (signal(SIGALRM, Watchdog) == SIG_IGN)
+                            signal(SIGALRM, SIG_IGN);
+                         }
+                      }
                     UserShutdown = false;
                     continue; // skip the rest of the housekeeping for now
                     }
@@ -996,6 +1168,8 @@ int main(int argc, char *argv[])
               PluginManager.Housekeeping();
               }
            }
+        // Main thread hooks of plugins:
+        PluginManager.MainThreadHook();
         }
   if (Interrupted)
      isyslog("caught signal %d", Interrupted);
@@ -1018,7 +1192,8 @@ Exit:
      Setup.Save();
      }
   cDevice::Shutdown();
-  PluginManager.Shutdown();
+  PluginManager.Shutdown(true);
+  cSchedules::Cleanup(true);
   ReportEpgBugFixStats();
   if (WatchdogTimeout > 0)
      dsyslog("max. latency time %d seconds", MaxLatencyTime);
